from PyQt5 import QtGui, QtWidgets
from PyQt5.QtCore import Qt,QObject, pyqtSlot, QThread, pyqtSignal, QLocale, QDateTime, QSize, QByteArray, QRectF, QPointF

import sys
from pymodaq.daq_utils.plotting.viewer1D.viewer1D_main import Viewer1D
from pymodaq.daq_utils.plotting.viewer2D.viewer2D_main import Viewer2D


from collections import OrderedDict
import numpy as np

from pyqtgraph.parametertree import Parameter, ParameterTree
import pymodaq.daq_utils.daq_utils as utils

from pyqtgraph import LinearRegionItem
import copy

from pymodaq.daq_utils.plotting.viewerND.signal import Signal
import datetime


class ViewerND(QtWidgets.QWidget, QObject):
    """

        ======================== =========================================
        **Attributes**            **Type**

        *dockarea*                instance of pyqtgraph.DockArea
        *mainwindow*              instance of pyqtgraph.DockArea
        *title*                   string
        *waitime*                 int
        *x_axis*                  float array
        *y_axis*                  float array
        *data_buffer*             list of data
        *ui*                      QObject
        ======================== =========================================

        Raises
        ------
        parent Exception
            If parent argument is None in constructor abort

        See Also
        --------
        set_GUI


        References
        ----------
        PyQt5, pyqtgraph, QtWidgets, QObject

    """
    command_DAQ_signal=pyqtSignal(list)
    log_signal=pyqtSignal(str)
    data_to_export_signal=pyqtSignal(OrderedDict) #edict(name=self.DAQ_type,data0D=None,data1D=None,data2D=None)


    def __init__(self,parent=None):
        QLocale.setDefault(QLocale(QLocale.English, QLocale.UnitedStates))
        super(ViewerND, self).__init__()
        # if parent is None:
        #     raise Exception('no valid parent container, expected dockarea')
            # parent=DockArea()
            # exit(0)

        if parent is None:
            parent = QtWidgets.QWidget()
            parent.show()
        self.parent = parent

        self.wait_time = 2000
        self.viewer_type = 'DataND'  # ☺by default but coul dbe used for 3D visualization

        self.x_axis = dict(data=None, label='', units='')
        self.y_axis = dict(data=None, label='', units='')

        self.data_buffer = []  # convenience list to store 0D data to be displayed
        self.datas = None
        self.datas_settings = None
        # set default data shape case
        self.axes_nav = None
        self.data_axes = None
        # self.set_nav_axes(3)
        self.ui = QObject()  # the user interface
        self.set_GUI()




    @pyqtSlot(OrderedDict)
    def export_data(self, datas):
        datas['acq_time_s'] = datetime.datetime.now().timestamp()
        self.data_to_export_signal.emit(datas)

    def get_data_dimension(self):
        try:
            dimension="("
            for ind,ax in enumerate(self.datas.axes_manager.navigation_shape):
                if ind!=len(self.datas.axes_manager.navigation_shape)-1:
                    dimension+=str(ax)+','
                else:
                    dimension+=str(ax)+'|'
            for ind, ax in enumerate(self.datas.axes_manager.signal_shape):
                if ind!=len(self.datas.axes_manager.signal_shape)-1:
                    dimension+=str(ax)+','
                else:
                    dimension+=str(ax)+')'
            return dimension
        except Exception as e:
            self.update_status(utils.getLineInfo()+str(e), self.wait_time)
            return ""


    def parameter_tree_changed(self,param,changes):
        """
            Foreach value changed, update :
                * Viewer in case of **DAQ_type** parameter name
                * visibility of button in case of **show_averaging** parameter name
                * visibility of naverage in case of **live_averaging** parameter name
                * scale of axis **else** (in 2D pymodaq type)

            Once done emit the update settings signal to link the commit.

            =============== =================================== ================================================================
            **Parameters**    **Type**                           **Description**
            *param*           instance of ppyqtgraph parameter   the parameter to be checked
            *changes*         tuple list                         Contain the (param,changes,info) list listing the changes made
            =============== =================================== ================================================================

            See Also
            --------
            change_viewer, daq_utils.custom_parameter_tree.iter_children
        """
        try:
            for param, change, data in changes:
                path = self.settings.childPath(param)
                if path is not None:
                    childName = '.'.join(path)
                else:
                    childName = param.name()
                if change == 'childAdded':
                    pass

                elif change == 'value':
                    pass
                    #if param.name()=='navigator_axes':
                    #    self.update_data_signal()
                    #    self.settings.child('data_shape_settings', 'data_shape').setValue(str(datas_transposed))
                    #    self.set_data(self.datas)


        except Exception as e:
            self.update_status(utils.getLineInfo()+str(e),self.wait_time,'log')

    def set_axis(self,Npts):
        """
            | Set axis values from node and a linspace regular distribution

            ================ ======================= ==========================================
            **Parameters**     **Type**                **Description**

             *node*           tables Group instance   the root node of the local treated tree
            ================ ======================= ==========================================

            Returns
            -------
            float array
                the computed values axis.

        """
        axis=np.linspace(0,Npts,Npts,endpoint=False)
        return axis

    def set_data(self,datas_transposed,temp_data=False, **kwargs):
        """
        """
        try:
            self.nav_x_axis = dict(data=None, label='', units='')
            self.nav_y_axis = dict(data=None, label='', units='')
            if len(self.axes_nav)==1 or len(self.axes_nav)==2:#1D Navigator
                self.nav_y_axis['data'] = [0]
                if 'nav_x_axis' in kwargs:
                    if not isinstance(kwargs['nav_x_axis'], dict):
                        self.nav_x_axis['data'] = kwargs['nav_x_axis'][:]
                    else:
                        self.nav_x_axis = copy.deepcopy(kwargs['nav_x_axis'])
                else:
                    self.nav_x_axis['data']=self.set_axis(datas_transposed.axes_manager.navigation_shape[0])


            if len(self.axes_nav)==2:#2D Navigator:
                if 'nav_y_axis' in kwargs:
                    if not isinstance(kwargs['nav_y_axis'], dict):
                        self.nav_y_axis['data'] = kwargs['nav_y_axis'][:]
                    else:
                        self.nav_y_axis = copy.deepcopy(kwargs['nav_y_axis'])
                else:
                    self.nav_y_axis['data']=self.set_axis(datas_transposed.axes_manager.navigation_shape[1])


            ##########################################################################
            #display the correct signal viewer
            if len(datas_transposed.axes_manager.signal_shape)==0: #signal data are 0D
                self.ui.viewer1D.parent.setVisible(True)
                self.ui.viewer2D.parent.setVisible(False)
            elif len(datas_transposed.axes_manager.signal_shape)==1: #signal data are 1D
                self.ui.viewer1D.parent.setVisible(True)
                self.ui.viewer2D.parent.setVisible(False)
            elif len(datas_transposed.axes_manager.signal_shape)==2: #signal data are 2D
                self.ui.viewer1D.parent.setVisible(False)
                self.ui.viewer2D.parent.setVisible(True)
            self.x_axis = dict(data=None, label='', units='')
            self.y_axis = dict(data=None, label='', units='')
            if len(datas_transposed.axes_manager.signal_shape)==1 or len(datas_transposed.axes_manager.signal_shape)==2:#signal data are 1D
                if 'x_axis' in kwargs:
                    if not isinstance(kwargs['x_axis'], dict):
                        self.x_axis['data'] = kwargs['x_axis'][:]
                        self.x_axis = kwargs['x_axis']
                    else:
                        self.x_axis = copy.deepcopy(kwargs['x_axis'])
                else:
                    self.x_axis['data']=self.set_axis(datas_transposed.axes_manager.signal_shape[0])

            if len(datas_transposed.axes_manager.signal_shape)==2:#signal data is 2D
                if 'y_axis' in kwargs:
                    if not isinstance(kwargs['y_axis'], dict):
                        self.y_axis['data'] = kwargs['y_axis'][:]
                        self.y_axis = kwargs['y_axis']
                    else:
                        self.y_axis = copy.deepcopy(kwargs['y_axis'])
                else:
                    self.y_axis['data']=self.set_axis(datas_transposed.axes_manager.signal_shape[1])

            if len(self.axes_nav)==0 or len(self.axes_nav)==1:
                self.update_viewer_data(*self.ui.navigator1D.ui.crosshair.get_positions())
            elif len(self.axes_nav)==2:
                self.update_viewer_data(*self.ui.navigator2D.ui.crosshair.get_positions())


            ##get ROI bounds from viewers if any
            ROI_bounds_1D=[]
            try:
                self.ROI1D.getRegion()
                indexes_values = utils.find_index(self.ui.viewer1D.x_axis, self.ROI1D.getRegion())
                ROI_bounds_1D.append(QPointF(indexes_values[0][0],indexes_values[1][0]))
            except:
                pass

            ROI_bounds_2D=[]
            try:
                ROI_bounds_2D.append(QRectF(self.ROI2D.pos().x(),self.ROI2D.pos().y(),
                                        self.ROI2D.size().x(),self.ROI2D.size().y()))
            except:
                pass



            #############################################################
            #display the correct navigator viewer and set some parameters
            if len(self.axes_nav)==0:#no Navigator
                self.ui.navigator1D.parent.setVisible(False)
                self.ui.navigator2D.parent.setVisible(False)
                self.navigator_label.setVisible(False)
                self.ROI1D.setVisible(False)
                self.ROI2D.setVisible(False)
                navigator_data=[]


            elif len(self.axes_nav)==1:#1D Navigator
                self.ROI1D.setVisible(True)
                self.ROI2D.setVisible(True)
                self.ui.navigator1D.parent.setVisible(True)
                self.ui.navigator2D.parent.setVisible(False)
                self.navigator_label.setVisible(True)
                self.ui.navigator1D.remove_plots()
                self.ui.navigator1D.x_axis=self.nav_x_axis

                if len(datas_transposed.axes_manager.signal_shape) == 0: #signal data are 0D
                    navigator_data=[datas_transposed.data]

                elif len(datas_transposed.axes_manager.signal_shape) == 1:#signal data are 1D
                    if ROI_bounds_1D!=[]:
                        if self.ui.combomath.currentText() == 'Sum':
                            navigator_data=[datas_transposed.isig[pt.x():pt.y()+1].sum((-1)).data for pt in ROI_bounds_1D]
                        elif self.ui.combomath.currentText() == 'Mean':
                            navigator_data = [datas_transposed.isig[pt.x():pt.y() + 1].mean((-1)).data for pt in
                                              ROI_bounds_1D]
                        elif self.ui.combomath.currentText() == 'Half-life':
                            navigator_data = [datas_transposed.isig[pt.x():pt.y() + 1].halflife((-1)).data for pt in
                                              ROI_bounds_1D]
                    else:
                        if self.ui.combomath.currentText() == 'Sum':
                            navigator_data=[datas_transposed.isig[:].sum((-1)).data]
                        elif self.ui.combomath.currentText() == 'Mean':
                            navigator_data = [datas_transposed.isig[:].mean((-1)).data]
                        elif self.ui.combomath.currentText() == 'Half-life':
                            navigator_data = [datas_transposed.isig[:].halflife((-1)).data]

                elif len(datas_transposed.axes_manager.signal_shape)==2:#signal data is 2D
                    if ROI_bounds_2D!=[]:
                        navigator_data=[datas_transposed.isig[rect.x():rect.x()+rect.width(),rect.y():rect.y()+rect.height()].sum((-1,-2)).data for rect in ROI_bounds_2D]
                    else:
                        navigator_data=[datas_transposed.sum((-1,-2)).data]

                else:
                    pass
                if temp_data:
                    self.ui.navigator1D.show_data_temp(navigator_data)
                else:
                    self.ui.navigator1D.show_data(navigator_data)

            elif len(self.axes_nav)==2:#2D Navigator:
                self.ROI1D.setVisible(True)
                self.ROI2D.setVisible(True)
                self.ui.navigator1D.parent.setVisible(False)
                self.ui.navigator2D.parent.setVisible(True)
                self.navigator_label.setVisible(True)
                self.ui.navigator2D.x_axis = self.nav_x_axis
                self.ui.navigator2D.y_axis = self.nav_y_axis

                if len(datas_transposed.axes_manager.signal_shape)==0: #signal data is 0D
                    navigator_data=[datas_transposed.data]

                elif len(datas_transposed.axes_manager.signal_shape)==1: #signal data is 1D
                    if ROI_bounds_1D!=[]:
                        if self.ui.combomath.currentText() == 'Sum':
                            navigator_data=[datas_transposed.isig[pt.x():pt.y()+1].sum((-1)).data for pt in ROI_bounds_1D]
                        elif self.ui.combomath.currentText() == 'Mean':
                            navigator_data = [datas_transposed.isig[pt.x():pt.y() + 1].mean((-1)).data for pt in
                                              ROI_bounds_1D]
                        elif self.ui.combomath.currentText() == 'Half-life':
                            navigator_data = [datas_transposed.isig[pt.x():pt.y() + 1].halflife((-1)).data for pt in
                                              ROI_bounds_1D]
                    else:
                        if self.ui.combomath.currentText() == 'Sum':
                            navigator_data=[datas_transposed.isig[:].sum((-1)).data]
                        elif self.ui.combomath.currentText() == 'Mean':
                            navigator_data = [datas_transposed.isig[:].mean((-1)).data]
                        elif self.ui.combomath.currentText() == 'Half-life':
                            navigator_data = [datas_transposed.isig[:].halflife((-1)).data]

                elif len(datas_transposed.axes_manager.signal_shape)==2: #signal data is 2D
                    if ROI_bounds_2D!=[]:
                        navigator_data=[datas_transposed.isig[rect.x():rect.x()+rect.width(),rect.y():rect.y()+rect.height()].sum((-1,-2)).data for rect in ROI_bounds_2D]
                    else:
                        navigator_data=[datas_transposed.sum((-1,-2)).data]


                else:
                    pass
                if temp_data:
                    self.ui.navigator2D.setImageTemp(*navigator_data)
                else:
                    self.ui.navigator2D.setImage(*navigator_data)


            else:
                raise Exception('No valid Navigator shape')



        except Exception as e:
            self.update_status(utils.getLineInfo()+str(e),self.wait_time,'log')

    def set_data_test(self,data_shape='3D'):


        x=utils.linspace_step(0,20,1)
        y=utils.linspace_step(0,30,1)
        t=utils.linspace_step(0,200,1)
        z=utils.linspace_step(0,200,1)
        datas=np.zeros((len(y),len(x),len(t),len(z)))
        amp=utils.gauss2D(x,7,5,y,12,10)
        for indx in range(len(x)):
            for indy in range(len(y)):
                datas[indy,indx,:,:]=amp[indy,indx]*(utils.gauss2D(z,50+indx*2,20,t,50+3*indy,30)+np.random.rand(len(t),len(z))/10)

        if data_shape=='4D':
            self.show_data(datas,temp_data=False,nav_axes=[2,3])
        elif data_shape=='3D':
            self.show_data(np.sum(datas,axis=3),temp_data=False,nav_axes=[0,1])
        elif data_shape=='2D':
            self.show_data(np.sum(datas,axis=(2,3)))
        elif data_shape=='1D':
            self.show_data(np.sum(datas,axis=(1,2,3)))

    def set_nav_axes(self,Ndim,nav_axes=None):
        self.data_axes=[ind for ind in range(Ndim)]
        if nav_axes is None:
            if Ndim>0:
                self.axes_nav=self.data_axes[0:2]
            else:
                self.axes_nav=self.data_axes[0]
        else:
            self.axes_nav=[Ndim-ind_ax-1 for ind_ax in nav_axes] #because hyperspy array order is reversed compared to numpy....
        self.axes_signal=[ax for ax in self.data_axes if ax not in self.axes_nav]
        self.settings.child('data_shape_settings', 'navigator_axes').setValue(dict(all_items=[str(ax) for ax in self.data_axes],selected=[str(ax) for ax in self.axes_nav]))


    def set_GUI(self):
        """

        """
        #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        main_layout=QtWidgets.QGridLayout()
        self.parent.setLayout(main_layout)

        Vsplitter=QtWidgets.QSplitter(Qt.Vertical)
        #Hsplitter=QtWidgets.QSplitter(Qt.Horizontal)


        #%%create status bar
        self.ui.statusbar=QtWidgets.QStatusBar()
        self.ui.statusbar.setMaximumHeight(25)
        Vsplitter.addWidget(self.ui.statusbar)




        params=[
            {'title': 'set data:', 'name': 'set_data_4D', 'type': 'action', 'visible': False},
            {'title': 'set data:', 'name': 'set_data_3D', 'type': 'action', 'visible': False},
            {'title': 'set data:', 'name': 'set_data_2D', 'type': 'action', 'visible': False},
            {'title': 'set data:', 'name': 'set_data_1D', 'type': 'action', 'visible': False},
            {'title': 'Signal shape', 'name': 'data_shape_settings', 'type': 'group', 'children': [
                    {'title': 'Data shape:', 'name': 'data_shape', 'type': 'str', 'value': "", 'readonly': True},
                    {'title': 'Navigator axes:','name': 'navigator_axes', 'type': 'itemselect'},
                    {'title': 'Set Nav axes:', 'name': 'set_nav_axes', 'type': 'action', 'visible': True},
                        ]},
                ]

        self.settings=Parameter.create(name='Param', type='group', children=params)
        ##self.signal_axes_selection()

        #connecting from tree
        self.settings.sigTreeStateChanged.connect(self.parameter_tree_changed)#any changes on the settings
        self.settings.child(('set_data_1D')).sigActivated.connect(lambda : self.set_data_test('1D'))
        self.settings.child(('set_data_2D')).sigActivated.connect(lambda : self.set_data_test('2D'))
        self.settings.child(('set_data_3D')).sigActivated.connect(lambda : self.set_data_test('3D'))
        self.settings.child(('set_data_4D')).sigActivated.connect(lambda : self.set_data_test('4D'))
        self.settings.child('data_shape_settings','set_nav_axes').sigActivated.connect(self.update_data)
        ##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        ##% 1D signalviewer
        viewer1D_widget=QtWidgets.QWidget()
        self.ui.viewer1D=Viewer1D(viewer1D_widget)
        self.ROI1D = LinearRegionItem()
        self.ui.viewer1D.viewer.plotwidget.plotItem.addItem(self.ROI1D)
        self.ui.combomath = QtWidgets.QComboBox()
        self.ui.combomath.addItems(['Sum', 'Mean', 'Half-life'])
        self.ui.viewer1D.ui.horizontalLayout.insertWidget(4, self.ui.combomath)
        self.ui.combomath.currentIndexChanged.connect(self.update_Navigator)

        self.ROI1D.sigRegionChangeFinished.connect(self.update_Navigator)
        #% 2D viewer Dock
        viewer2D_widget=QtWidgets.QWidget()
        self.ui.viewer2D=Viewer2D(viewer2D_widget)
        self.ui.viewer2D.ui.auto_levels_pb.click()
        self.ui.viewer2D.ui.ROIselect_pb.click()
        self.ROI2D = self.ui.viewer2D.ui.ROIselect
        self.ui.viewer2D.ROI_select_signal.connect(self.update_Navigator)

        ###dock_viewer=Dock('Signal')
        ###dock_viewer.addWidget(viewer1D_widget)
        ###dock_viewer.addWidget(viewer2D_widget)


        ##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        ##% Navigator viewer Dock
        navigator1D_widget=QtWidgets.QWidget()
        self.ui.navigator1D=Viewer1D(navigator1D_widget)
        self.ui.navigator1D.ui.crosshair.crosshair_dragged.connect(self.update_viewer_data)
        self.ui.navigator1D.ui.crosshair_pb.click()
        self.ui.navigator1D.data_to_export_signal.connect(self.export_data)
        navigator2D_widget=QtWidgets.QWidget()
        self.ui.navigator2D=Viewer2D(navigator2D_widget)
        self.ui.navigator2D.ui.auto_levels_pb.click()
        self.ui.navigator2D.crosshair_dragged.connect(self.update_viewer_data) #export scaled position in conjonction with 2D scaled axes
        self.ui.navigator2D.ui.crosshair_pb.click()
        self.ui.navigator2D.data_to_export_signal.connect(self.export_data)


        widg0=QtWidgets.QWidget()
        VLayout0=QtWidgets.QVBoxLayout()
        self.navigator_label=QtWidgets.QLabel('Navigation View')
        self.navigator_label.setMaximumHeight(15)
        VLayout0.addWidget(self.navigator_label)
        VLayout0.addWidget(navigator1D_widget)
        VLayout0.addWidget(navigator2D_widget)
        widg0.setLayout(VLayout0)
        Vsplitter.insertWidget(0,widg0)

        widg1=QtWidgets.QWidget()
        VLayout1=QtWidgets.QVBoxLayout()
        self.viewer_label=QtWidgets.QLabel('Data View')
        self.viewer_label.setMaximumHeight(15)
        VLayout1.addWidget(self.viewer_label)
        VLayout1.addWidget(viewer1D_widget)
        VLayout1.addWidget(viewer2D_widget)
        widg1.setLayout(VLayout1)
        Vsplitter.insertWidget(1,widg1)


        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/cartesian.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.ui.set_signals_pb_1D=QtWidgets.QPushButton('')
        self.ui.set_signals_pb_1D.setToolTip('Change navigation/signal axes')
        self.ui.set_signals_pb_1D_bis = QtWidgets.QPushButton('')
        self.ui.set_signals_pb_1D_bis.setToolTip('Change navigation/signal axes')
        self.ui.set_signals_pb_1D.setIcon(icon)
        self.ui.set_signals_pb_1D_bis.setIcon(icon)
        self.ui.set_signals_pb_2D=QtWidgets.QPushButton('')
        self.ui.set_signals_pb_2D.setToolTip('Change navigation/signal axes')
        self.ui.set_signals_pb_2D.setIcon(icon)
        self.ui.set_signals_pb_2D_bis = QtWidgets.QPushButton('')
        self.ui.set_signals_pb_2D_bis.setToolTip('Change navigation/signal axes')
        self.ui.set_signals_pb_2D_bis.setIcon(icon)

        self.ui.navigator1D.ui.horizontalLayout.insertWidget(0,self.ui.set_signals_pb_1D)
        self.ui.navigator2D.ui.horizontalLayout_2.insertWidget(0,self.ui.set_signals_pb_2D)
        self.ui.viewer1D.ui.horizontalLayout.insertWidget(0, self.ui.set_signals_pb_1D_bis)
        self.ui.viewer2D.ui.horizontalLayout_2.insertWidget(0, self.ui.set_signals_pb_2D_bis)

        main_layout.addWidget(Vsplitter)

        self.ui.set_signals_pb_1D.clicked.connect(self.signal_axes_selection)
        self.ui.set_signals_pb_2D.clicked.connect(self.signal_axes_selection)
        self.ui.set_signals_pb_1D_bis.clicked.connect(self.signal_axes_selection)
        self.ui.set_signals_pb_2D_bis.clicked.connect(self.signal_axes_selection)

        #to start: display as default a 2D navigator and a 1D viewer
        self.ui.navigator1D.parent.setVisible(False)
        self.ui.viewer2D.parent.setVisible(True)


    def show_data_temp(self,datas,nav_axes=None, **kwargs):
        """
        """
        self.show_data(datas,temp_data=True,nav_axes=nav_axes, **kwargs)


    def show_data(self,datas,temp_data=False,nav_axes=None, **kwargs):
        """Display datas as a hyperspaced dataset
        only one numpy ndarray should be used
        """
        self.data_buffer = []
        try:
            if self.data_axes is not None:
                if datas.ndim != len(self.data_axes) or self.axes_nav != nav_axes:
                    self.set_nav_axes(datas.ndim, nav_axes) #init the list of axes and set the preset to nav_axes
            else:
                self.set_nav_axes(datas.ndim, nav_axes) #init the list of axes and set the preset to nav_axes

            #self.datas=hs.signals.BaseSignal(datas)
            self.datas = Signal(datas)
            self.datas_settings = kwargs
            self.update_data_signal()

            self.settings.child('data_shape_settings', 'data_shape').setValue(self.get_data_dimension())
            self.set_data(self.datas, temp_data=temp_data, **kwargs)

        except Exception as e:
            self.update_status(utils.getLineInfo()+str(e),self.wait_time,'log')


    def signal_axes_selection(self):
        self.ui.settings_tree=ParameterTree()
        self.ui.settings_tree.setMinimumWidth(300)
        self.ui.settings_tree.setParameters(self.settings, showTop=False)
        self.signal_axes_widget = QtWidgets.QWidget()
        layout=QtWidgets.QVBoxLayout()
        self.signal_axes_widget .setLayout(layout)
        layout.addWidget(self.ui.settings_tree)
        self.signal_axes_widget.adjustSize()
        self.signal_axes_widget.show()

    def update_data(self):
        self.update_data_signal()
        self.settings.child('data_shape_settings', 'data_shape').setValue(self.get_data_dimension())
        self.set_data(self.datas)

    def update_data_signal(self):
        try:
            self.axes_nav=[int(ax) for ax in self.settings.child('data_shape_settings', 'navigator_axes').value()['selected']]
            self.axes_signal=[ax for ax in self.data_axes if ax not in self.axes_nav]


            self.datas=self.datas.transpose(signal_axes=self.axes_signal,navigation_axes=self.axes_nav)


        except Exception as e:
            self.update_status(utils.getLineInfo()+str(e),self.wait_time,'log')

    def update_Navigator(self):
        ##self.update_data_signal()
        self.set_data(self.datas, **self.datas_settings)


    def update_status(self,txt,wait_time=1000,log=''):
        """
            | Update the statut bar showing a Message with a delay of wait_time ms (1s by default)

            ================ ======== ===========================
            **Parameters**   **Type**     **Description**

             *txt*            string   the text message to show

             *wait_time*      int      the delay time of showing
            ================ ======== ===========================

        """
        self.ui.statusbar.showMessage(txt,wait_time)
        if log=='log':
            self.log_signal.emit(txt)


    def update_viewer_data(self,posx=0,posy=0):
        """
            |PyQt5 slot triggered by the crosshair signal from the 1D or 2D Navigator
            | Update the viewer informations from an x/y given position and store data.
            | Ruled by the viewer type (0D,1D,2D)

            ================ ========= ==============================
            **Parameters**   **Type**        **Description**

             *posx*           int       the x position of the viewer

             *posy*           int       the y position of the viewer
            ================ ========= ==============================

            See Also
            --------
            update_status
        """
        try:
            #datas_transposed=self.update_data_signal(self.datas)
            if len(self.axes_nav)==0:
                data=self.datas.data

            elif len(self.axes_nav)==1:
                ind_x=utils.find_index(self.nav_x_axis['data'],posx)[0][0]
                data=self.datas.inav[ind_x].data
            elif len(self.axes_nav)==2:
                ind_x=utils.find_index(self.nav_x_axis['data'],posx)[0][0]
                ind_y=utils.find_index(self.nav_y_axis['data'],posy)[0][0]
                data=self.datas.inav[ind_x,ind_y].data

            if len(self.datas.axes_manager.signal_shape)==0:#means 0D data, plot on 1D viewer
                self.data_buffer.extend(data)
                self.ui.viewer1D.show_data([self.data_buffer])

            elif len(self.datas.axes_manager.signal_shape)==1: #means 1D data, plot on 1D viewer
                self.ui.viewer1D.remove_plots()
                self.ui.viewer1D.x_axis=self.x_axis
                self.ui.viewer1D.show_data([data])

            elif len(self.datas.axes_manager.signal_shape)==2: #means 2D data, plot on 2D viewer
                self.ui.viewer2D.x_axis = self.x_axis
                self.ui.viewer2D.y_axis = self.y_axis
                self.ui.viewer2D.setImage(data)
        except Exception as e:
            self.update_status(utils.getLineInfo()+str(e),wait_time=self.wait_time)



if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    form = QtWidgets.QWidget()

    prog = ViewerND()
    prog.settings.child(('set_data_4D')).show(True)
    prog.settings.child(('set_data_3D')).show(True)
    prog.settings.child(('set_data_2D')).show(True)
    prog.settings.child(('set_data_1D')).show(True)
    prog.signal_axes_selection()
    form.show()
    sys.exit(app.exec_())


from PyQt5 import QtGui, QtWidgets
from PyQt5.QtCore import Qt,QObject, pyqtSlot, QThread, pyqtSignal, QLocale, QDateTime, QSize

import sys
import pyqtgraph as pg
from pyqtgraph.parametertree import Parameter, ParameterTree
import pyqtgraph.parametertree.parameterTypes as pTypes
import pymodaq.daq_utils.custom_parameter_tree as custom_tree
from easydict import EasyDict as edict
import socket, select
import numpy as np
from pymodaq.daq_utils.daq_utils import gauss1D, gauss2D, check_received_length, check_sended, message_to_bytes,\
    get_int, get_list, send_string, send_list, get_array, get_string
from collections import OrderedDict
from pymodaq.daq_utils.daq_utils import ThreadCommand, ScanParameters, getLineInfo
from pymodaq.daq_utils.tcp_server_client import TCPServer, tcp_parameters

comon_parameters=[{'title': 'Controller Status:', 'name': 'controller_status', 'type': 'list', 'value': 'Master', 'values': ['Master','Slave']},]


class DAQ_Viewer_base(QObject):
    """
        ===================== ===================================
        **Attributes**          **Type**
        *hardware_averaging*    boolean
        *data_grabed_signal*    instance of pyqtSignal
        *params*                list
        *settings*              instance of pyqtgraph Parameter
        *parent*                ???
        *status*                dictionnary
        ===================== ===================================

        See Also
        --------
        send_param_status
    """
    hardware_averaging=False
    data_grabed_signal=pyqtSignal(list)
    data_grabed_signal_temp=pyqtSignal(list)

    params = []
    def __init__(self,parent=None,params_state=None):
        QObject.__init__(self)
        self.parent_parameters_path = [] #this is to be added in the send_param_status to take into account when the current class instance parameter list is a child of some other class
        self.settings = Parameter.create(name='Settings', type='group', children=self.params)
        if params_state is not None:
            if isinstance(params_state, dict):
                self.settings.restoreState(params_state)
            elif isinstance(params_state, Parameter):
                self.settings.restoreState(params_state.saveState())


        self.settings.sigTreeStateChanged.connect(self.send_param_status)

        self.parent = parent
        self.status = edict(info="",controller=None,initialized=False)
        self.scan_parameters = None

    def get_axis(self):
        if '1D' in str(type(self)) or '2D' in str(type(self)):
            self.emit_x_axis()

        if '2D'  in str(type(self)):
            self.emit_y_axis()

    def emit_status(self,status):
        """
            Emit the status signal from the given status.

            =============== ============ =====================================
            **Parameters**    **Type**     **Description**
            *status*                       the status information to transmit
            =============== ============ =====================================
        """
        if self.parent is not None:
            self.parent.status_sig.emit(status)
            QtWidgets.QApplication.processEvents()
        else:
            print(*status)

    @pyqtSlot(ScanParameters)
    def update_scanner(self, scan_parameters):
        self.scan_parameters = scan_parameters

    def update_com(self):
        """
        If some communications settings have to be reinit
        Returns
        -------

        """
        pass

    @pyqtSlot(edict)
    def update_settings(self, settings_parameter_dict):
        """
            Update the settings tree from settings_parameter_dict.
            Finally do a commit to activate changes.
            
            ========================== ============= =====================================================
            **Parameters**              **Type**      **Description**
            *settings_parameter_dict*   dictionnnary  a dictionnary listing path and associated parameter
            ========================== ============= =====================================================

            See Also
            --------
            send_param_status, commit_settings
        """
            # settings_parameter_dict=edict(path=path,param=param)
        try:
            path = settings_parameter_dict['path']
            param = settings_parameter_dict['param']
            change = settings_parameter_dict['change']
            try:
                self.settings.sigTreeStateChanged.disconnect(self.send_param_status)
            except: pass
            if change == 'value':
                self.settings.child(*path[1:]).setValue(param.value()) #blocks signal back to main UI
            elif change == 'childAdded':
                child = Parameter.create(name = 'tmp')
                child.restoreState(param)
                self.settings.child(*path[1:]).addChild(child) #blocks signal back to main UI
                param = child

            elif change == 'parent':
                children = custom_tree.get_param_from_name(self.settings, param.name())

                if children is not None:
                    path = custom_tree.get_param_path(children)
                    self.settings.child(*path[1:-1]).removeChild(children)

            self.settings.sigTreeStateChanged.connect(self.send_param_status)

            self.commit_settings(param)
        except Exception as e:
            self.emit_status(ThreadCommand("Update_Status",[str(e),'log']))

    def commit_settings(self,param):
        """
            Not implemented.
        """
        pass

    def stop(self):
        pass

    def set_spectro_wl(self, spectro_wl):
        """
        Particular case if the plugin is a spectrometer
        Parameters
        ----------
        spectro_wl

        Returns
        -------

        """
        pass


    def send_param_status(self,param,changes):
        """
            Check for changes in the given (parameter,change,information) tuple list.
            In case of value changed, send the 'update_settings' ThreadCommand with concerned path,data and change as attributes.

            =============== ============================================ ============================
            **Parameters**    **Type**                                    **Description**
            *param*           instance of pyqtgraph parameter             The parameter to check
            *changes*         (parameter,change,information) tuple list   The changes list to course
            =============== ============================================ ============================

            See Also
            --------
            daq_utils.ThreadCommand
        """
        for param, change, data in changes:
            path = self.settings.childPath(param)
            if change == 'childAdded':
                #first create a "copy" of the actual parameter and send this "copy", to be restored in the main UI
                self.emit_status(ThreadCommand('update_settings',[self.parent_parameters_path+path, [data[0].saveState(), data[1]], change])) #send parameters values/limits back to the GUI. Send kind of a copy back the GUI otherwise the child reference will be the same in both th eUI and the plugin so one of them will be removed

            elif change == 'value' or change == 'limits' or change == 'options':
                self.emit_status(ThreadCommand('update_settings', [self.parent_parameters_path+path, data, change])) #send parameters values/limits back to the GUI
            elif change == 'parent':
                pass

            pass

    def emit_x_axis(self):
        """
            Convenience function
            Emit the thread command "x_axis" with x_axis as an attribute.

            See Also
            --------
            daq_utils.ThreadCommand
        """
        self.emit_status(ThreadCommand("x_axis",[self.x_axis]))

    def emit_y_axis(self):
        """
            Emit the thread command "y_axis" with y_axis as an attribute.

            See Also
            --------
            daq_utils.ThreadCommand
        """
        self.emit_status(ThreadCommand("y_axis",[self.y_axis]))
 

class DAQ_Viewer_TCP_server(DAQ_Viewer_base, TCPServer):
    """
        ================= ==============================
        **Attributes**      **Type**
        *command_server*    instance of pyqtSignal
        *x_axis*            1D numpy array
        *y_axis*            1D numpy array
        *data*              double precision float array
        ================= ==============================

        See Also
        --------
        utility_classes.DAQ_TCP_server
    """
    params_GRABBER =[] #parameters of a client grabber
    command_server=pyqtSignal(list)

    message_list=["Quit","Send Data 0D","Send Data 1D","Send Data 2D","Status","Done","Server Closed","Info","Infos",
                  "Info_xml", 'x_axis', 'y_axis']
    socket_types=["GRABBER"]
    params= comon_parameters+tcp_parameters

    def __init__(self,parent=None,params_state=None, grabber_type='2D'):
        """

        Parameters
        ----------
        parent
        params_state
        grabber_type: (str) either '0D', '1D' or '2D'
        """
        self.client_type = "GRABBER"
        DAQ_Viewer_base.__init__(self, parent,params_state) #initialize base class with commom attributes and methods
        TCPServer.__init__(self, self.client_type)

        self.x_axis = None
        self.y_axis = None
        self.data = None
        self.grabber_type = grabber_type
        self.ind_data=0
        self.data_mock=None

    def command_to_from_client(self,command):
        sock = self.find_socket_within_connected_clients(self.client_type)
        if sock is not None:  # if client self.client_type is connected then send it the command

            if command == 'x_axis':
                x_axis = dict(data=get_array(sock))
                x_axis['label'] = get_string(sock)
                x_axis['units'] = get_string(sock)
                self.x_axis = x_axis.copy()
                self.emit_x_axis()
            elif command == 'y_axis':
                y_axis = dict(data=get_array(sock))
                y_axis['label'] = get_string(sock)
                y_axis['units'] = get_string(sock)
                self.y_axis = y_axis.copy()
                self.emit_y_axis()

            else:
                self.send_command(sock, command)



        else:  # else simulate mock data
            if command == "Send Data 0D":
                self.set_1D_Mock_data()
                self.data_mock = np.array([self.data_mock[0]])
            elif command == "Send Data 1D":
                self.set_1D_Mock_data()
                data = self.data_mock
            elif command == "Send Data 2D":
                self.set_2D_Mock_data()
                data = self.data_mock
            self.process_cmds('Done')

    def send_data(self, sock, data):
        """
            To match digital and labview, send again a command.

            =============== ============================== ====================
            **Parameters**   **Type**                       **Description**
            *sock*                                          the socket receipt
            *data*           double precision float array   the data to be sent
            =============== ============================== ====================

            See Also
            --------
            send_command, check_send_data
        """
        self.send_command(sock, 'Done')

        if len(data.shape) == 0:
            Nrow = 1
            Ncol = 0
        elif len(data.shape) == 1:
            Nrow = data.shape[0]
            Ncol = 0
        elif len(data.shape) == 2:
            Nrow = data.shape[0]
            Ncol = data.shape[1]
        data_bytes = data.tobytes()
        check_sended(sock, np.array([len(data_bytes)],
                                    dtype='>i4').tobytes())  # first send length of data after reshaping as 1D bytes array
        check_sended(sock, np.array([Nrow], dtype='>i4').tobytes())  # then send dimension of lines
        check_sended(sock, np.array([Ncol], dtype='>i4').tobytes())  # then send dimensions of columns

        check_sended(sock, data_bytes)  # then send data

    def read_data(self,sock):
        """
            Read the unsigned 32bits int data contained in the given socket in five steps :
                * get back the message
                * get the list length
                * get the data length
                * get the number of row
                * get the number of column
                * get data

            =============== ===================== =========================
            **Parameters**    **Type**             **Description**
            *sock*              ???                the socket to be readed
            *dtype*           numpy unint 32bits   ???
            =============== ===================== =========================

            See Also
            --------
            check_received_length
        """

        data_list = get_list(sock, 'array')

        return data_list



    def data_ready(self,data):
        """
            Send the grabed data signal. to be written in the detailed plugin using this base class

        for instance:
        self.data_grabed_signal.emit([OrderedDict(name=self.client_type,data=[data], type='Data2D')])  #to be overloaded
        """
        pass

    def command_done(self, command_sock):
        try:
            sock = self.find_socket_within_connected_clients(self.client_type)
            if sock is not None:  # if client self.client_type is connected then send it the command
                data = self.read_data(sock)
            else:
                data = self.data_mock

            if command_sock is None:
                # self.data_grabed_signal.emit([OrderedDict(data=[data],name='TCP GRABBER', type='Data2D')]) #to be directly send to a viewer
                self.data_ready(data)
                # print(data)
            else:
                self.send_data(command_sock, data)  # to be send to a client

        except Exception as e:
            self.emit_status(ThreadCommand("Update_Status", [str(e), 'log']))

    def commit_settings(self,param):

        if param.name() in custom_tree.iter_children(self.settings.child(('infos')), []):
            grabber_socket = [client['socket'] for client in self.connected_clients if client['type'] == self.client_type][0]
            send_string(grabber_socket, 'set_info')

            path = custom_tree.get_param_path(param)[2:]#get the path of this param as a list starting at parent 'infos'
            send_list(grabber_socket, path)

            #send value
            data = custom_tree.parameter_to_xml_string(param)
            send_string(grabber_socket, data)

    def ini_detector(self, controller=None):
        """
            | Initialisation procedure of the detector updating the status dictionnary.
            |
            | Init axes from image , here returns only None values (to tricky to di it with the server and not really necessary for images anyway)

            See Also
            --------
            utility_classes.DAQ_TCP_server.init_server, get_xaxis, get_yaxis
        """
        self.status.update(edict(initialized=False,info="",x_axis=None,y_axis=None,controller=None))
        try:
            self.settings.child(('infos')).addChildren(self.params_GRABBER)

            self.init_server()

        #%%%%%%% init axes from image , here returns only None values (to tricky to di it with the server and not really necessary for images anyway)
            self.x_axis=self.get_xaxis()
            self.y_axis=self.get_yaxis()
            self.status.x_axis=self.x_axis
            self.status.y_axis=self.y_axis
            self.status.initialized=True
            self.status.controller=self.serversocket
            return self.status

        except Exception as e:
            self.status.info=getLineInfo()+ str(e)
            self.status.initialized=False
            return self.status

    def close(self):
        """
            Should be used to uninitialize hardware.

            See Also
            --------
            utility_classes.DAQ_TCP_server.close_server
        """
        self.listening=False
        self.close_server()

    def get_xaxis(self):
        """
            Obtain the horizontal axis of the image.

            Returns
            -------
            1D numpy array
                Contains a vector of integer corresponding to the horizontal camera pixels.
        """
        pass
        return self.x_axis

    def get_yaxis(self):
        """
            Obtain the vertical axis of the image.

            Returns
            -------
            1D numpy array
                Contains a vector of integer corresponding to the vertical camera pixels.
        """
        pass
        return self.y_axis

    def grab_data(self, Naverage=1, **kwargs):
        """
            Start new acquisition.
            Grabbed indice is used to keep track of the current image in the average.

            ============== ========== ==============================
            **Parameters**   **Type**  **Description**

            *Naverage*        int       Number of images to average
            ============== ========== ==============================

            See Also
            --------
            utility_classes.DAQ_TCP_server.process_cmds
        """
        try:
            self.ind_grabbed=0 #to keep track of the current image in the average
            self.Naverage=Naverage
            self.process_cmds("Send Data {:s}".format(self.grabber_type))
            #self.command_server.emit(["process_cmds","Send Data 2D"])


        except Exception as e:
            self.emit_status(ThreadCommand('Update_Status',[getLineInfo()+ str(e),"log"]))

    def stop(self):
        """
            not implemented.
        """
        pass
        return ""


    def set_1D_Mock_data(self):
        self.data_mock
        x=np.linspace(0,99,100)
        data_tmp=10*gauss1D(x,50,10,1)+1*np.random.rand((100))
        self.ind_data+=1
        self.data_mock=np.roll(data_tmp,self.ind_data)


    def set_2D_Mock_data(self):
        self.x_axis=np.linspace(0,50,50,endpoint=False)
        self.y_axis=np.linspace(0,30,30,endpoint=False)
        self.data_mock=10*gauss2D(self.x_axis,20,10,
                                  self.y_axis,15,7,1)+2*np.random.rand(len(self.y_axis),len(self.x_axis))


if __name__ == '__main__':
    prog = DAQ_TCP_server()


import sys
import tables
from collections import OrderedDict
import datetime
import os
import numpy as np

from PyQt5 import QtGui, QtWidgets
from PyQt5.QtCore import QObject, pyqtSlot, pyqtSignal, QLocale, QDateTime, QRectF, QDate, QThread, Qt

from pyqtgraph.dockarea import Dock
from pymodaq.daq_utils.daq_utils import DockArea

from pyqtgraph.parametertree import Parameter, ParameterTree
import pyqtgraph.parametertree.parameterTypes as pTypes
import pymodaq.daq_utils.custom_parameter_tree as custom_tree
from pymodaq.daq_utils.daq_utils import select_file, Enm2cmrel, Ecmrel2Enm, nm2eV, eV2nm, eV2radfs, l2w, getLineInfo, ThreadCommand
from pymodaq.daq_utils.plotting.qled import QLED

from pymodaq.daq_viewer.daq_viewer_main import DAQ_Viewer
from pymodaq.daq_move.daq_move_main import DAQ_Move
from pymodaq.daq_utils.plotting.viewer0D.viewer0D_main import Viewer0D
from pymodaq.daq_utils.plotting.viewer1D.viewer1D_main import Viewer1D
from pymodaq.daq_utils.plotting.viewer2D.viewer2D_main import Viewer2D
from pymodaq.daq_utils.h5browser import browse_data
from pymodaq.daq_utils import daq_utils as utils
from pymodaq.daq_utils.h5browser import H5Browser


class Spectrometer(QObject):
    """
    Defines a Spectrometer object, unified interface for many spectrometers

    Parameters that could be set in the selected detector plugin (should be defined there):
    'laser_wl' : value of the configured laser (could eventually be changed, case of Xplora, Labram...)
    'spectro_center_freq': value of the configured grating center wavelength (could eventually be changed, case of Shamrock, Xplora...)


    """
    #custom signal that will be fired sometimes. Could be connected to an external object method or an internal method
    log_signal = pyqtSignal(str)

    #list of dicts enabling the settings tree on the user interface
    params = [{'title': 'Configuration settings:', 'name': 'config_settings', 'type': 'group', 'children': [
                        {'title': 'Laser wavelength (nm):', 'name': 'laser_wl', 'type': 'float', 'value': 515.},
                        {'title': 'Laser wavelength (nm):', 'name': 'laser_wl_list', 'type': 'list', 'values':['']},
                        {'title': 'List of detectors:', 'name': 'list_detector', 'type': 'list', 'values':['']},
                        {'title': 'Show detector:', 'name': 'show_det', 'type': 'bool', 'value': False},
                        ],},
              {'title': 'Calibration settings:', 'name': 'calib_settings', 'type': 'group', 'children': [
                     ]},
              {'title': 'Acquisition settings:', 'name': 'acq_settings', 'type': 'group', 'children': [
                  {'title': 'Spectro. Center:', 'name': 'spectro_center_freq', 'type': 'float', 'value': 800, },
                  {'title': 'Units:', 'name': 'units', 'type': 'list', 'value': 'nm', 'values': ['nm', 'cm-1', 'eV']},
                  {'title': 'Exposure (ms):', 'name': 'exposure', 'type': 'float', 'value': 100, },
              ]},
              ]


    def __init__(self, parent):
        QLocale.setDefault(QLocale(QLocale.English, QLocale.UnitedStates))
        super().__init__()
        if not isinstance(parent, DockArea):
            raise Exception('no valid parent container, expected a DockArea')

        self.wait_time = 2000 #ms

        self.dockarea = parent
        self.mainwindow = parent.parent()
        self.spectro_widget = QtWidgets.QWidget()

        """
        List of the possible plugins that could be used with Spectrometer module
        type : dimensionality of the detector
        name: name of the plugin
        calib = True means there is a builtin calibration of the frequency axis
        movable : tells if the dispersion can be set (for instance by moving a grating)
        unit: valid only if calib is True. Unit of the calibration axis (x_axis of the detector), most often in
              nanometers. Possible values are 'nm', 'radfs' (rad/femtosecond), 'eV'
        laser: if False,  laser cannot be changed by the program, do it manually
        laser_list: if laser is True, laser_list gives a list of selectable lasers
        """
        self.detectors = [ dict(type='DAQ1D', name='Mock', calib=True, movable=True, unit='nm', laser=False, laser_list=[]),
                                dict(type='DAQ1D', name='OceanOptics', calib=True, movable=False, unit='nm', laser=False, laser_list=[]),
                                dict(type='DAQ1D', name='Shamrock', calib=True, movable=True, unit='nm', laser=False, laser_list=[]),
                                dict(type='DAQ1D', name='TCPServer', calib=False, movable=False, unit=None, laser=False, laser_list=[]),
                                dict(type='DAQ2D', name='AndorCCD', calib=False, movable=False, unit= None, laser=False, laser_list=[]),]
        self.current_det = None #will be after initialization one of these dictionaries



        #init the object parameters
        self.detector = None
        self.save_file_pathname = None
        self._spectro_wl = 550 # center wavelngth of the spectrum
        self.viewer_freq_axis = dict(data=None, label='Photon energy', units='')
        self.raw_data = []

        #init the user interface
        self.set_GUI()

        dets = [det['name'] for det in self.detectors]
        self.settings.child('config_settings', 'list_detector').setOpts(limits=dets)
        self.settings.child('config_settings', 'list_detector').setValue('Mock')
        self.show_detector(False)



    def set_GUI(self):
        ###########################################
        ###########################################
        #init the docks containing the main widgets

        #######################################################################################################################
        #create a dock containing a viewer object, displaying the data for the spectrometer
        self.dock_viewer = Dock('Viewer dock', size=(350, 350))
        self.dockarea.addDock(self.dock_viewer, 'left')
        target_widget = QtWidgets.QWidget()
        self.viewer = Viewer1D(target_widget)
        self.dock_viewer.addWidget(target_widget)

        ###################################################################################
        #create 2 docks to display the DAQ_Viewer (one for its settings, one for its viewer)
        dock_detector_settings = Dock("Detector Settings", size=(350, 350))
        self.dockarea.addDock(dock_detector_settings, 'right')
        dock_detector_settings.float()
        self.detector_area = dock_detector_settings.area
        dock_detector = Dock("Detector Viewer", size=(350, 350))
        self.detector_area.addDock(dock_detector, 'right', dock_detector_settings)
        #init one daq_viewer object named detector
        self.detector = DAQ_Viewer(self.dockarea, dock_settings=dock_detector_settings,
                                                 dock_viewer=dock_detector, title="Spectro Detector", DAQ_type='DAQ1D')
        self.detector.log_signal.connect(self.update_status)
        self.detector.init_signal[bool].connect(self.initialized)
        self.detector.custom_sig[ThreadCommand].connect(self.cmd_from_det)
        ################################################################
        #create a logger dock where to store info senf from the programm
        self.dock_logger = Dock("Logger")
        self.logger_list = QtWidgets.QListWidget()
        self.logger_list.setMinimumWidth(300)
        self.dock_logger.addWidget(self.logger_list)
        self.dockarea.addDock(self.dock_logger, 'right')
        self.log_signal[str].connect(self.add_log)



        ############################################
        # creating a menubar
        self.menubar = self.mainwindow.menuBar()
        self.create_menu(self.menubar)

        #creating a toolbar
        self.toolbar = QtWidgets.QToolBar()
        self.create_toolbar()
        self.mainwindow.addToolBar(self.toolbar)

        #creating a status bar
        self.statusbar = QtWidgets.QStatusBar()
        self.statusbar.setMaximumHeight(25)
        self.status_center = custom_tree.SpinBoxCustom()
        self.status_center.setReadOnly(True)
        self.status_center.setButtonSymbols(QtWidgets.QAbstractSpinBox.NoButtons)
        self.status_center.setMaximumWidth(100)
        self.status_center.setToolTip('center frequency of the spectrum, either in nm or cm-1')

        self.status_init = QLED()
        self.status_init.setToolTip('Initialization state of the detector')
        self.status_init.set_as_false()

        self.statusbar.addPermanentWidget(self.status_center)
        self.statusbar.addPermanentWidget(self.status_init)
        self.dockarea.window().setStatusBar(self.statusbar)


        #############################################
        self.settings = Parameter.create(name='settings', type='group', children=self.params)
        self.settings.sigTreeStateChanged.connect(self.parameter_tree_changed)

        dock_config_settings = Dock('Configuration', size=(300, 350))
        self.dockarea.addDock(dock_config_settings, 'above', self.dock_logger)
        # create main parameter tree
        self.config_settings_tree = ParameterTree()
        dock_config_settings.addWidget(self.config_settings_tree, 10)
        self.config_settings_tree.setMinimumWidth(300)
        self.config_settings_tree.setParameters(self.settings.child(('config_settings')), showTop=False)
        #any change to the tree on the user interface will call the parameter_tree_changed method where all actions will be applied

        dock_calib_settings = Dock('Calibration', size=(300, 350))
        self.dockarea.addDock(dock_calib_settings, 'above', self.dock_logger)
        # create main parameter tree
        self.calib_settings_tree = ParameterTree()
        dock_calib_settings.addWidget(self.calib_settings_tree, 10)
        self.calib_settings_tree.setMinimumWidth(300)
        self.calib_settings_tree.setParameters(self.settings.child(('calib_settings')), showTop=False)
        #any change to the tree on the user interface will call the parameter_tree_changed method where all actions will be applied



        #this one for the custom application settings
        dock_acq_settings = Dock('Acquisition', size=(300, 350))
        self.dockarea.addDock(dock_acq_settings, 'above', dock_config_settings)
        # create main parameter tree
        self.acq_settings_tree = ParameterTree()
        dock_acq_settings.addWidget(self.acq_settings_tree, 10)
        self.acq_settings_tree.setMinimumWidth(300)
        self.acq_settings_tree.setParameters(self.settings.child(('acq_settings')), showTop=False)



        self.status_center.setSuffix(self.settings.child('acq_settings', 'units').value())
        self.status_center.setValue(self.settings.child('acq_settings', 'spectro_center_freq').value())

    @pyqtSlot(ThreadCommand)
    def cmd_from_det(self,status):
        if status.command == 'spectro_wl':
            self.spectro_wl_is(status.attributes[0])


    def update_status(self,txt, wait_time=1000, log_type=None):
        """

        """
        self.statusbar.showMessage(txt,wait_time)
        if log_type is not None:
            self.log_signal.emit(txt)


    def init_detector(self):
        self.settings.child('config_settings', 'list_detector').setOpts(readonly=self.init_action.isChecked())
        if self.init_action.isChecked():
            self.statusbar.showMessage('Initializing')

            self.detector.ui.IniDet_pb.click()
            if self.detector.ui.Detector_type_combo.currentText() == 'Mock':
                self.detector.settings.child('main_settings', 'wait_time').setValue(100)
            QtWidgets.QApplication.processEvents()
            QThread.msleep(1000)
            self.detector.grab_done_signal.connect(self.show_data)
            self.statusbar.clearMessage()

            #set current detector
            det_index = [det['name'] for det in self.detectors].index(self.detector.ui.Detector_type_combo.currentText())
            self.current_det = self.detectors[det_index]

            #apply current detector particularities
            self.settings.child('acq_settings', 'spectro_center_freq').setOpts(readonly=not self.current_det['movable'])
            self.get_spectro_wl()

            if self.current_det['laser']:
                self.settings.child('config_settings', 'laser_wl_list').show()
                self.settings.child('config_settings', 'laser_wl').setOpts(readonly=True)
                self.settings.child('config_settings', 'laser_wl_list').setOpts(limits=self.current_det['laser_list'])
            else:
                self.settings.child('config_settings', 'laser_wl').setOpts(readonly=False)
                self.settings.child('config_settings', 'laser_wl_list').hide()

        else:
            self.detector.ui.IniDet_pb.click()

            #self.detector.grab_done_signal.disconnect()

    @pyqtSlot(bool)
    def initialized(self, state):
        self.grab_action.setEnabled(state)
        self.snap_action.setEnabled(state)
        if state:
            self.status_init.set_as_true()

        else:
            self.status_init.set_as_false()

    def update_center_frequency(self, spectro_wl):
        self._spectro_wl = spectro_wl
        if self.settings.child('acq_settings', 'units').value() == 'nm':
            self.settings.child('acq_settings', 'spectro_center_freq').setValue(spectro_wl)
        elif self.settings.child('acq_settings', 'units').value() == 'cm-1':
            self.settings.child('acq_settings', 'spectro_center_freq').setValue(Enm2cmrel(spectro_wl,
                                                                                          self.settings.child(
                                                                                              'config_settings',
                                                                                              'laser_wl')))
        elif self.settings.child('acq_settings', 'units').value() == 'eV':
            self.settings.child('acq_settings', 'spectro_center_freq').setValue(nm2eV(spectro_wl))

        self.status_center.setSuffix(self.settings.child('acq_settings', 'units').value())
        self.status_center.setValue(self.settings.child('acq_settings', 'spectro_center_freq').value())

    @pyqtSlot(float)
    def spectro_wl_is(self, spectro_wl):
        """
        this slot receives a signal from the detector telling it what's the current spectro_wl
        Parameters
        ----------
        spectro_wl
        """
        self._spectro_wl = spectro_wl
        self.update_center_frequency(spectro_wl)


    def set_spectro_wl(self, spectro_wl):
        if self.current_det['movable']:
            self.detector.command_detector.emit(ThreadCommand('set_spectro_wl', [spectro_wl]))

    def get_spectro_wl(self):
        if self.current_det['calib']:
            self.detector.command_detector.emit(ThreadCommand('get_spectro_wl'))

    @property
    def spectro_wl(self):
        # try to get the param value from detector (if it has been added in the plugin)
        return self._spectro_wl

    @spectro_wl.setter
    def spectro_wl(self, spec_wl):
        # try to get the param value from detector (if it has been added in the plugin)
        self.set_spectro_wl(spec_wl)


    def get_freq_axis(self):
        try:
            if self.current_det['calib']:
                #try to get the  calibration axis
                if self.current_det['unit'] == 'nm':
                    x_axis = self.detector.ui.viewers[0].x_axis
                elif self.current_det['unit'] == 'radfs':
                    x_axis = l2w(self.detector.ui.viewers[0].x_axis)  # converts rad/fs to nanometers
                elif self.current_det['unit'] == 'eV':
                    x_axis = eV2nm(self.detector.ui.viewers[0].x_axis)  # converts eV to nanometers
                else:
                    self.update_status('calibration axis unit cannot be used', 'log')

                self.update_center_frequency(np.median(x_axis))

            else:
                x_axis = None
                self.update_center_frequency(None)


            self.viewer_freq_axis.update(dict(data=x_axis, units=self.current_det['unit']))
        except Exception as e:
            self.update_status('Cannot get the current center frequency', self.wait_time, 'log')

    def show_detector(self, show=True):
        self.detector_area.window().setVisible(show)


    def parameter_tree_changed(self, param, changes):
        for param, change, data in changes:
            path = self.settings.childPath(param)
            if path is not None:
                childName = '.'.join(path)
            else:
                childName = param.name()
            if change == 'childAdded':
                pass

            elif change == 'value':
                if param.name() == 'list_detector':
                    if data is not None:
                        dets = [det['name'] for det in self.detectors]
                        ind = dets.index(data)
                        det_type = self.detectors[ind]['type']
                        self.detector.ui.DAQ_type_combo.setCurrentText(det_type)
                        QtWidgets.QApplication.processEvents()

                        self.detector.ui.Detector_type_combo.setCurrentText(data)

                elif param.name() == 'show_det':
                    self.show_detector(data)

                elif param.name() == 'spectro_center_freq':
                    unit = self.settings.child('acq_settings', 'units').value()
                    if unit == 'nm':
                        center_wavelength = data
                    elif unit == 'cm-1':
                        center_wavelength = Ecmrel2Enm(data, self.settings.child( 'config_settings', 'laser_wl').value())
                    elif unit == 'eV':
                        center_wavelength = eV2nm(data)

                    if self.spectro_wl != center_wavelength:
                        self.spectro_wl = center_wavelength



                elif param.name() == 'units':
                    if data == 'nm':
                        self.settings.child('acq_settings', 'spectro_center_freq').setValue(self._spectro_wl)
                    elif data == 'cm-1':
                        self.settings.child('acq_settings', 'spectro_center_freq').setValue(Enm2cmrel(self._spectro_wl,
                                                self.settings.child( 'config_settings', 'laser_wl').value()))
                    elif data == 'eV':
                        self.settings.child('acq_settings', 'spectro_center_freq').setValue(nm2eV(self._spectro_wl))

                    self.status_center.setSuffix(self.settings.child('acq_settings', 'units').value())
                    self.status_center.setValue(self.settings.child('acq_settings', 'spectro_center_freq').value())

                elif param.name() == 'laser_wl_list':
                    self.settings.child('config_settings', 'laser_wl').setValue(data)

                elif param.name() == 'laser_wl':
                    self.move_laser_wavelength(data)


            elif change == 'parent':
                pass


    def move_laser_wavelength(self, laser_wavelength):
        #do hardware stuff if possible (labspec...)
        try:
            if self.current_det['laser']:
                if 'laser_wl' in custom_tree.iter_children(self.detector.settings,[]):
                    self.statusbar.showMessage('Moving laser wavelength', self.wait_time)
                    index_laser = custom_tree.iter_children(self.detector.settings,[]).index('laser_wl')
                    param_laser = custom_tree.iter_children_params(self.detector.settings, [])[index_laser]

                    param_laser.setValue(laser_wavelength)
        except Exception as e:
            self.update_status(getLineInfo()+ str(e),self.wait_time,'log')

    @pyqtSlot(OrderedDict)
    def show_data(self,data):
        """
        do stuff with data from the detector if its grab_done_signal has been connected
        Parameters
        ----------
        data: (OrderedDict) #OrderedDict(name=self.title,x_axis=None,y_axis=None,z_axis=None,data0D=None,data1D=None,data2D=None)
        """

        if 'data1D' in data:
            self.raw_data = []
            for key in data['data1D']:
                self.raw_data.append(data['data1D'][key]['data'])
                if 'x_axis' in data['data1D'][key]:
                    x_axis = data['data1D'][key]['x_axis']

                    if np.any(x_axis['data'] != self.viewer_freq_axis['data']):
                        self.viewer_freq_axis.update(x_axis)

            self.viewer.show_data(self.raw_data)
            if np.any(self.viewer_freq_axis['data'] != self.viewer.x_axis):
                self.viewer.x_axis = self.viewer_freq_axis.copy()











    def create_menu(self, menubar):
        """
        """
        menubar.clear()

        #%% create file menu
        file_menu=menubar.addMenu('File')
        load_action=file_menu.addAction('Load file')
        load_action.triggered.connect(self.load_file)
        save_action=file_menu.addAction('Save file')
        save_action.triggered.connect(self.save_data)

        file_menu.addSeparator()
        quit_action=file_menu.addAction('Quit')
        quit_action.triggered.connect(self.quit_function)

        settings_menu=menubar.addMenu('Settings')
        docked_menu=settings_menu.addMenu('Docked windows')
        action_load=docked_menu.addAction('Load Layout')
        action_save=docked_menu.addAction('Save Layout')


    def load_file(self):
        #init the data browser module
        widg = QtWidgets.QWidget()
        self.data_browser = H5Browser(widg)
        widg.show()

    def quit_function(self):
        #close all stuff that need to be
        self.detector.quit_fun()
        QtWidgets.QApplication.processEvents()
        self.mainwindow.close()

    def create_toolbar(self):
        iconquit = QtGui.QIcon()
        iconquit.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/close2.png"), QtGui.QIcon.Normal,
                           QtGui.QIcon.Off)
        self.quit_action = QtWidgets.QAction(iconquit, "Quit program", None)
        self.toolbar.addAction(self.quit_action)
        self.quit_action.triggered.connect(self.quit_function)

        iconload = QtGui.QIcon()
        iconload.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/Open.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.loadaction = QtWidgets.QAction(iconload, "Load target file (.h5, .png, .jpg) or data from camera", None)
        self.toolbar.addAction(self.loadaction)
        self.loadaction.triggered.connect(self.load_file)


        iconsave = QtGui.QIcon()
        iconsave.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/SaveAs.png"), QtGui.QIcon.Normal,
                           QtGui.QIcon.Off)
        self.saveaction = QtWidgets.QAction(iconsave, "Save current data", None)
        self.toolbar.addAction(self.saveaction)
        self.saveaction.triggered.connect(self.save_data)

        self.init_action = QtWidgets.QAction("Ini. det", None)
        self.init_action.setCheckable(True)
        self.toolbar.addAction(self.init_action)
        self.init_action.triggered.connect(self.init_detector)

        iconrun = QtGui.QIcon()
        iconrun.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/run2.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.grab_action = QtWidgets.QAction(iconrun, 'Grab', None)
        self.grab_action.setCheckable(True)
        self.toolbar.addAction(self.grab_action)
        self.grab_action.triggered.connect(self.grab_detector)

        iconsnap = QtGui.QIcon()
        iconsnap.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/snap.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.snap_action = QtWidgets.QAction(iconsnap, 'Snap', None)
        self.snap_action.triggered.connect(self.snap_detector)
        self.toolbar.addAction(self.snap_action)

        self.grab_action.setEnabled(False)
        self.snap_action.setEnabled(False)


    def grab_detector(self):
        self.detector.ui.grab_pb.click()

    def snap_detector(self):
        self.detector.ui.single_pb.click()


    def save_data(self):
        try:
            fname = utils.select_file(start_path=self.save_file_pathname, save=True, ext='h5')

            if not (not (fname)):
                with tables.open_file(str(fname), mode='w', title='an example h5 file name') as h5file:
                    data_to_save = np.squeeze(np.array(self.raw_data))
                    #save metadata
                    h5file.root._v_attrs['settings'] = custom_tree.parameter_to_xml_string(self.settings)
                    for ind in range(data_to_save.shape[1]):
                        arr = h5file.create_array('/', 'data_{:d}'.format(ind), data_to_save[:,ind])
                        arr._v_attrs['shape'] = data_to_save.shape[0]
                        arr._v_attrs['type'] = 'data1D'

                    arr = h5file.create_array('/', 'data_2D', data_to_save)
                    arr._v_attrs['shape'] = data_to_save.shape
                    arr._v_attrs['type'] = 'data2D'



                    logger = "logging"
                    text_atom = tables.atom.ObjectAtom()
                    logger_array = h5file.create_vlarray('/', logger, atom=text_atom)
                    logger_array._v_attrs['type'] = 'list'
                    for ind_log in range(self.logger_list.count()):
                        txt = self.logger_list.item(ind_log).text()
                        logger_array.append(txt)

                st = 'file {:s} has been saved'.format(fname)
                self.add_log(st)
                self.settings.child('min_settings', 'info').setValue(st)


        except Exception as e:
            self.add_log(str(e))


    @pyqtSlot(str)
    def add_log(self, txt):
        """
            Add a log to the logger list from the given text log and the current time

            ================ ========= ======================
            **Parameters**   **Type**   **Description**

             *txt*             string    the log to be added
            ================ ========= ======================

        """
        now = datetime.datetime.now()
        new_item = QtWidgets.QListWidgetItem(str(now) + ": " + txt)
        self.logger_list.addItem(new_item)
        ##to do
        ##self.save_parameters.logger_array.append(str(now)+": "+txt)

    @pyqtSlot(str)
    def emit_log(self, txt):
        """
            Emit a log-signal from the given log index

            =============== ======== =======================
            **Parameters**  **Type** **Description**

             *txt*           string   the log to be emitted
            =============== ======== =======================

        """
        self.log_signal.emit(txt)


if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    win = QtWidgets.QMainWindow()
    area = DockArea()
    win.setCentralWidget(area)
    win.resize(1000, 500)
    win.setWindowTitle('pymodaq example')
    prog = Spectrometer(area)
    win.show()
    sys.exit(app.exec_())


from PyQt5 import QtGui, QtWidgets, QtCore
from PyQt5.QtCore import Qt,QObject, pyqtSlot, QThread, pyqtSignal, QLocale, QTimer, QDateTime, QDate, QTime

import sys
import os
from collections import OrderedDict
import numpy as np
import tables

from pyqtgraph.dockarea import Dock, DockArea
from pyqtgraph.parametertree import Parameter, ParameterTree
from pyqtgraph import HistogramLUTWidget
import pymodaq.daq_utils.custom_parameter_tree as custom_tree# to be placed after importing Parameter
from pymodaq.daq_utils.h5browser import browse_data
from pymodaq.daq_utils.plotting.viewer2D.viewer2d_basic import Viewer2DBasic, ImageItem
from pymodaq.daq_utils import daq_utils as utils

from pymodaq.daq_utils.daq_utils import get_set_local_dir
local_path = get_set_local_dir()
navigator_path= os.path.join(local_path, 'navigator_temp_files')
if not os.path.isdir(navigator_path):
    os.makedirs(navigator_path)

Ntick = 128
colors_red =np.array([(int(r),0,0) for r in np.linspace(0,255,Ntick)])
colors_green=np.array([(0,int(g),0) for g in np.linspace(0,255,Ntick)])
colors_blue=np.array([(0,0,int(b)) for b in np.linspace(0,255,Ntick)])

class Navigator(QObject):
    log_signal = pyqtSignal(str)
    sig_double_clicked = pyqtSignal(float, float)

    def __init__(self,parent=None, h5file=None):
        super(Navigator,self).__init__(parent)

        if parent is None:
            parent = QtWidgets.QWidget()
        self.parent = parent
        self.title = 'Navigator'
        self.status_time = 1000
        self.x_range = []
        self.y_range = []
        self.filters = tables.Filters(complevel=5)
        self.next_scan_index = 0
        self.viewer = None
        self.overlays=[]#%list of imageItem items displaying 2D scans info
        self.h5file_path = None
        self.h5file = h5file
        if h5file is not None:
            self.h5file_path = h5file.filename
            self.settings.child('settings', 'filepath').setValue(h5file.filename)
            self.settings.child('settings', 'Load h5').hide()
            self.show_overlay()

        self.setupUI()

    def create_toolbar(self):
        iconload = QtGui.QIcon()
        iconload.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/OpenLayers.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.loadaction = QtWidgets.QAction(iconload, "Load scan file (.h5)", None)
        self.toolbar.addAction(self.loadaction)
        self.loadaction.triggered.connect(self.load_data)

        iconloadim = QtGui.QIcon()
        iconloadim.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/Open_File_32.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.loadactionim = QtWidgets.QAction(iconloadim, "Load image file (.h5)", None)
        self.toolbar.addAction(self.loadactionim)
        self.loadactionim.triggered.connect(self.load_image)

        icon_ratio = QtGui.QIcon()
        icon_ratio.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/Zoom_1_1.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.action_ratio = QtWidgets.QAction(icon_ratio, "Set viewbox aspect ratio to 1", None)
        self.action_ratio.setCheckable(True)
        self.toolbar.addAction(self.action_ratio)
        self.action_ratio.triggered.connect(self.set_aspect_ratio)

        icon_moveat = QtGui.QIcon()
        icon_moveat.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/move_contour.png"), QtGui.QIcon.Normal,
                           QtGui.QIcon.Off)
        self.moveat_action = QtWidgets.QAction(icon_moveat, "When selected, double clicking on viewbox will move DAQ_Move modules", None)
        self.moveat_action.setCheckable(True)
        self.toolbar.addAction(self.moveat_action)


        icon_sel_all = QtGui.QIcon()
        icon_sel_all.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/select_all2.png"), QtGui.QIcon.Normal,
                           QtGui.QIcon.Off)
        self.sel_all_action = QtWidgets.QAction(icon_sel_all, "Select (show) all 2D scans on the viewer", None)
        self.toolbar.addAction(self.sel_all_action)
        self.sel_all_action.triggered.connect(self.show_all)

        icon_sel_none = QtGui.QIcon()
        icon_sel_none.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/select_none.png"), QtGui.QIcon.Normal,
                           QtGui.QIcon.Off)
        self.sel_none_action = QtWidgets.QAction(icon_sel_none, "Unselect (hide) all 2D scans on the viewer", None)
        self.toolbar.addAction(self.sel_none_action)
        self.sel_none_action.triggered.connect(self.show_none)

        icon_histo = QtGui.QIcon()
        icon_histo.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/Histogram.png"), QtGui.QIcon.Normal,
                           QtGui.QIcon.Off)
        self.histo_action = QtWidgets.QAction(icon_histo, "Show (hide) histograms", None)
        self.toolbar.addAction(self.histo_action)
        self.histo_action.setCheckable(True)
        self.histo_action.triggered.connect(self.show_histo)

    @pyqtSlot(float, float)
    def move_at(self, posx, posy):
        if self.moveat_action.isChecked():
            self.sig_double_clicked.emit(posx, posy)

    def show_histo(self):
        show_state = self.histo_action.isChecked()
        self.viewer.histo_widget.setVisible(show_state)

    def show_all(self):
        self.show_scans()

    def show_none(self):
        self.show_scans(False)

    def show_scans(self, show=True):
        for child in self.settings.child('scans'):
            val = child.value()
            val['checked'] = show
            child.setValue(val)
            child.sigValueChanged.emit(child, val)


    def list_2Dscans(self):
        try:
            scans=utils.get_h5file_scans(self.h5file)
            #settings=[dict(scan_name=node._v_name,path=node._v_pathname, pixmap=nparray2Qpixmap(node.read()))),...]
            params=[]
            for child in self.settings.child(('scans')).children():
                if 'Scan' in child.name():
                    self.settings.child(('scans')).removeChild(child)
            for scan in scans:
                params.append({'name': scan['scan_name'], 'type': 'pixmap_check', 'value':dict(data=scan['data'],checked=False,path=scan['path'])})
            self.settings.child(('scans')).addChildren(params)

            for child in self.settings.child(('scans')).children():
                val = child.value()
                val.update(dict(checked = True))
                child.setValue(val)
                child.sigValueChanged.emit(child, child.value())

        except Exception as e:
            self.update_status(utils.getLineInfo()+str(e),status_time=self.status_time,log_type='log')

    def load_image(self):
        #image_filepath = str(utils.select_file(start_path=None, save=False, ext='h5'))
        data, fname, node_path = browse_data(ret_all=True)
        if data is not None and fname != '':
            self.h5file_image = tables.open_file(fname)
            node = self.h5file_image.get_node(node_path)
            pixmaps = utils.get_h5file_scans(self.h5file_image, node._v_parent)
            self.settings.child('settings', 'imagepath').setValue(fname)
            other_child = [child for child in self.settings.child(('scans')).children() if 'Scan' not in child.name()]
            if len(other_child) >= 1:
                for child in other_child:
                    self.settings.child(('scans')).removeChild(child)
            params = []
            for pixmap in pixmaps:
                params.append({'name': pixmap['scan_name'], 'type': 'pixmap_check', 'value':dict(data=pixmap['data'],checked=False,path=pixmap['path'])})
            self.settings.child(('scans')).addChildren(params)

            val = self.settings.child('scans', pixmaps[0]['scan_name']).value()
            val.update(dict(checked = True))
            self.settings.child('scans', pixmaps[0]['scan_name']).setValue(val)
            self.settings.child('scans', pixmaps[0]['scan_name']).sigValueChanged.emit(self.settings.child('scans', pixmaps[0]['scan_name']),
                                                                                       self.settings.child('scans', pixmaps[0]['scan_name']).value())


    def load_data(self):
        self.h5file_path = str(utils.select_file(start_path=None,save=False, ext='h5'))
        if self.h5file_path != '':
            self.settings.child('settings', 'filepath').setValue(self.h5file_path)
            if self.h5file is not None:
                self.h5file.close()
            self.h5file = tables.open_file(self.h5file_path, 'a')
            self.list_2Dscans()

    def set_aspect_ratio(self):
        if self.action_ratio.isChecked():
            self.viewer.image_widget.plotitem.vb.setAspectLocked(lock=True, ratio=1)
        else:
            self.viewer.image_widget.plotitem.vb.setAspectLocked(lock=False, ratio=1)

    def settings_changed(self,param,changes):
        """
            Check for changes in the given (parameter,change,information) tuple list.
            In case of value changed, update the DAQscan_settings tree consequently.

            =============== ============================================ ==============================
            **Parameters**    **Type**                                     **Description**
            *param*           instance of pyqtgraph parameter              the parameter to be checked
            *changes*         (parameter,change,information) tuple list    the current changes state
            =============== ============================================ ==============================
        """
        for param, change, data in changes:
            path = self.settings.childPath(param)
            if path is not None:
                childName = '.'.join(path)
            else:
                childName = param.name()
            if change == 'childAdded':
                pass

            elif change == 'value':
                if param.parent().name() == 'scans':
                    if data['checked']:
                        try:
                            if 'Scan' in param.name():
                                h5file = self.h5file
                                nodes = list(h5file.walk_nodes(data['path']))
                            else:
                                h5file = self.h5file_image
                                nodes = [h5file.get_node(data['path'])]
                            ind = 0
                            for node in nodes:
                                flag = False
                                if 'type' in node._v_attrs and 'data_dimension' in node._v_attrs:
                                    if 'scan_type' in node._v_attrs:
                                        if node._v_attrs['scan_type'] == 'scan2D' and node._v_attrs['data_dimension'] == '0D': #2d scan of 0D data
                                            flag = True
                                        elif node._v_attrs['scan_type'] == '' and node._v_attrs['data_dimension'] == '2D': #image data (2D) with no scan
                                            flag = True

                                if flag:
                                    im=ImageItem()
                                    im.setOpacity(1)
                                    #im.setOpts(axisOrder='row-major')
                                    self.viewer.image_widget.plotitem.addItem(im)
                                    im.setCompositionMode(QtGui.QPainter.CompositionMode_Plus)
                                    im.setImage(node.read().T)

                                    if 'Scan' in param.name():
                                        x_axis = np.unique(h5file.get_node(data['path'])._v_parent._f_get_child(utils.capitalize('scan_x_axis')).read())
                                        y_axis = np.unique(h5file.get_node(data['path'])._v_parent._f_get_child(utils.capitalize('scan_y_axis')).read())
                                    else:
                                        x_axis = np.unique(h5file.get_node(data['path'])._v_parent._f_get_child(utils.capitalize('x_axis')).read())
                                        y_axis = np.unique(h5file.get_node(data['path'])._v_parent._f_get_child(utils.capitalize('y_axis')).read())


                                    dx = x_axis[1]-x_axis[0]
                                    dy = y_axis[1]-y_axis[0]
                                    im.setRect(QtCore.QRectF(np.min(x_axis),np.min(y_axis),np.max(x_axis)-np.min(x_axis)+dx,np.max(y_axis)-np.min(y_axis)+dy))
                                    if ind == 0:
                                        #im.setLookupTable(colors_red)
                                        self.viewer.histogram_red.setImageItem(im)
                                        if not self.viewer.histogram_red.isVisible():
                                            self.viewer.histogram_red.setVisible(True)
                                    elif ind == 1:
                                        #im.setLookupTable(colors_green)
                                        self.viewer.histogram_green.setImageItem(im)
                                        if not self.viewer.histogram_green.isVisible():
                                            self.viewer.histogram_green.setVisible(True)
                                    else:
                                        #im.setLookupTable(colors_blue)
                                        self.viewer.histogram_blue.setImageItem(im)
                                        if not self.viewer.histogram_blue.isVisible():
                                            self.viewer.histogram_blue.setVisible(True)

                                    self.overlays.append(dict(name='{:s}_{:03d}'.format(param.name(),ind),image=im))

                                    ind += 1
                            self.viewer.image_widget.view.autoRange()
                        except  Exception as e:
                            self.update_status(utils.getLineInfo()+str(e),status_time=self.status_time,log_type='log')

                    else:
                        for overlay in self.overlays[:]:
                            if param.name() in overlay['name']:
                                ind = self.overlays.index(overlay)
                                self.viewer.image_widget.plotitem.removeItem(overlay['image'])
                                self.overlays.pop(ind)

            elif change == 'parent':
                for overlay in self.overlays[:]:
                    if param.name() in overlay['name']:
                        ind = self.overlays.index(overlay)
                        self.viewer.image_widget.plotitem.removeItem(overlay['image'])
                        self.overlays.pop(ind)


    def setupUI(self):
        self.ui = QObject()
        layout =QtWidgets.QVBoxLayout()

        self.parent.setLayout(layout)
        sett_widget = QtWidgets.QWidget()
        self.sett_layout = QtWidgets.QHBoxLayout()
        sett_widget.setLayout(self.sett_layout)
        # creating a toolbar
        self.toolbar = QtWidgets.QToolBar()
        self.create_toolbar()
        layout.addWidget(self.toolbar)

        splitter = QtWidgets.QSplitter(Qt.Horizontal)
        layout.addWidget(splitter)



        #set viewer area
        widg = QtWidgets.QWidget()
        #self.viewer = Viewer2D(widg)
        self.viewer = Viewer2DBasic(widg)
        self.viewer.histogram_red.setVisible(False)
        self.viewer.histogram_green.setVisible(False)
        self.viewer.histogram_blue.setVisible(False)
        self.viewer.sig_double_clicked.connect(self.move_at)


        #displaying the scan list tree
        self.settings_tree = ParameterTree()
        #self.settings_tree.setMaximumWidth(300)
        self.settings_tree.setMinimumWidth(300)
        #self.settings_tree.setVisible(False)
        params_scan = [
                        {'title': 'Settings', 'name': 'settings', 'type': 'group', 'children': [
                            {'title': 'Load h5:', 'name': 'Load h5', 'type': 'action'},
                            {'title': 'h5 path:', 'name': 'filepath', 'type': 'str', 'value': '', 'readonly': True},
                            {'title': 'Load Image:', 'name': 'Load Image', 'type': 'action'},
                            {'title': 'Image path:', 'name': 'imagepath', 'type': 'str', 'value': '', 'readonly': True},
                        ]},
                        {'title': 'Scans', 'name': 'scans', 'type': 'group', 'children': []},
                        ]
        self.settings=Parameter.create(name='settings', type='group', children=params_scan)
        self.settings_tree.setParameters(self.settings, showTop=False)
        self.settings.sigTreeStateChanged.connect(self.settings_changed)

        self.settings.child('settings', 'Load h5').sigActivated.connect(self.load_data)
        self.settings.child('settings', 'Load Image').sigActivated.connect(self.load_image)

        self.ui.statusbar = QtWidgets.QStatusBar()
        self.ui.statusbar.setMaximumHeight(25)
        layout.addWidget(self.ui.statusbar)
        self.ui.log_message = QtWidgets.QLabel('Initializing')
        self.ui.statusbar.addPermanentWidget(self.ui.log_message)

        self.sett_layout.addWidget(self.settings_tree)
        splitter.addWidget(sett_widget)
        splitter.addWidget(self.viewer.parent)

    def show_image(self,data):
        """

        Parameters
        ----------
        data: (dict) with keys 'names', 'data', 'x_axis', 'y_axis', 'pixmap2D'
        """

        if self.h5file is None:
            scan_path, current_filename, dataset_path = utils.set_current_scan_path(navigator_path, base_name='Scan', update_h5=True, next_scan_index=self.next_scan_index,
                                  create_scan_folder=False)
            self.h5file = tables.open_file(str(dataset_path.joinpath(dataset_path.name+".h5")),'w')
            h5group = self.h5file.root
            data2D_group = self.h5file.create_group(h5group, 'Data2D')
            data2D_group._v_attrs.type = 'data2D'
        else:
            scan_path, current_filename, dataset_path = utils.set_current_scan_path(navigator_path, base_name='Scan',
                                                                                    update_h5=False, next_scan_index=self.next_scan_index,
                                                                                    create_scan_folder=False)
            if not self.h5file.isopen:
                self.h5file = tables.open_file(str(dataset_path.joinpath(dataset_path.name+".h5")), 'a')
        self.next_scan_index += 1
        curr_group = self.h5file.create_group('/Data2D', current_filename)
        curr_group._v_attrs['pixmap2D'] = data['pixmap2D']

        xarray = self.h5file.create_array(curr_group, "scan_x_axis_unique", obj=data['x_axis'], title=current_filename)
        xarray.attrs['shape'] = xarray.shape
        xarray.attrs['type'] = 'signal_axis'
        xarray.attrs['data_type'] = '1D'

        yarray = self.h5file.create_array(curr_group, "scan_y_axis_unique", obj=data['y_axis'], title=current_filename)
        yarray.attrs['shape'] = yarray.shape
        yarray.attrs['type'] = 'signal_axis'
        yarray.attrs['data_type'] = '1D'


        for ind_channel, name in enumerate(data['names']):
            try:
                channel_group = self.h5file.create_group(curr_group, name)
                channel_group._v_attrs.Channel_name = name
                array = self.h5file.create_carray(channel_group, current_filename + '_' + name,
                                                  obj=data['data'][ind_channel],
                                                  title='data',
                                                  filters=self.filters)
                array.attrs['type'] = 'data'
                array.attrs['data_type'] = '0D'
                array.attrs['data_name'] = name
                array.attrs['shape'] = data['data'][ind_channel].shape
                array.attrs['scan_type'] = 'Scan2D'
            except Exception as e:
                self.update_status(utils.getLineInfo()+str(e), status_time=self.status_time, log_type='log')

        self.update_2Dscans()

    def update_2Dscans(self):
        try:
            if not self.h5file.isopen:
                self.h5file = tables.open_file(self.h5file.filename, 'a')
            scans=utils.get_h5file_scans(self.h5file)
            #settings=[dict(scan_name=node._v_name,path=node._v_pathname, pixmap=nparray2Qpixmap(node.read()))),...]
            params=[]
            children = [child.name() for child in self.settings.child(('scans')).children()]
            for scan in scans:
                if scan['scan_name'] not in children:
                    params.append({'name': scan['scan_name'], 'type': 'pixmap_check', 'value':dict(data=scan['data'],checked=False,path=scan['path'])})
            self.settings.child(('scans')).addChildren(params)

            for child in self.settings.child(('scans')).children():
                if child.name() not in children:
                    val = child.value()
                    val.update(dict(checked = True))
                    child.setValue(val)
                    child.sigValueChanged.emit(child, child.value())

        except Exception as e:
            self.update_status(utils.getLineInfo()+str(e),status_time=self.status_time,log_type='log')

    def update_h5file(self, h5file):
        self.h5file = h5file
        self.update_2Dscans()

    def update_status(self,txt,status_time=0,log_type=None):
        """
            Show the txt message in the status bar with a delay of status_time ms.

            =============== =========== =======================
            **Parameters**    **Type**    **Description**
            *txt*             string      The message to show
            *status_time*       int         the delay of showing
            *log_type*        string      the type of the log
            =============== =========== =======================
        """
        try:
            self.ui.statusbar.showMessage(txt,status_time)
            if log_type is not None:
                self.log_signal.emit(self.title+': '+txt)
        except Exception as e:
            pass




if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    widg = QtWidgets.QWidget()
    prog = Navigator(widg)
    widg.show()
    sys.exit(app.exec_())


from PyQt5 import QtCore, QtGui, QtWidgets
from PyQt5.QtCore import QObject, pyqtSlot, QThread, pyqtSignal, QRectF, QRect, QPointF, QLocale
import sys
from collections import OrderedDict
import pyqtgraph as pg
import numpy as np
from easydict import EasyDict as edict


class Viewer2DBasic(QObject):
    sig_double_clicked = pyqtSignal(float, float)

    def __init__(self,parent=None, **kwargs):
        super(Viewer2DBasic,self).__init__()
        #setting the gui
        if parent is None:
            parent=QtWidgets.QWidget()
        self.parent = parent
        self.scaling_options = edict(scaled_xaxis=edict(label="",units=None,offset=0,scaling=1),scaled_yaxis=edict(label="",units=None,offset=0,scaling=1))
        self.setupUI()

    def scale_axis(self,xaxis,yaxis):
        return xaxis*self.scaling_options.scaled_xaxis.scaling+self.scaling_options.scaled_xaxis.offset,yaxis*self.scaling_options.scaled_yaxis.scaling+self.scaling_options.scaled_yaxis.offset

    @pyqtSlot(float,float)
    def double_clicked(self,posx,posy):
        self.sig_double_clicked.emit(posx,posy)

    def setupUI(self):
        vlayout = QtWidgets.QVBoxLayout()
        hsplitter = QtWidgets.QSplitter(QtCore.Qt.Horizontal)

        self.parent.setLayout(vlayout)
        vlayout.addWidget(hsplitter)
        self.ui = QObject()
        self.image_widget = ImageWidget()
        hsplitter.addWidget(self.image_widget)

        self.scaled_yaxis=AxisItem_Scaled('right')
        self.scaled_xaxis=AxisItem_Scaled('top')
        self.image_widget.plotitem.layout.addItem(self.scaled_xaxis, *(1,1))
        self.image_widget.plotitem.layout.addItem(self.scaled_yaxis, *(2,2))
        self.image_widget.view.sig_double_clicked.connect(self.double_clicked)
        self.scaled_xaxis.linkToView(self.image_widget.view)
        self.scaled_yaxis.linkToView(self.image_widget.view)

        # histograms
        self.histo_widget = QtWidgets.QWidget()
        histo_layout = QtWidgets.QHBoxLayout()
        self.histo_widget.setLayout(histo_layout)
        self.histogram_red = pg.HistogramLUTWidget()
        self.histogram_green = pg.HistogramLUTWidget()
        self.histogram_blue = pg.HistogramLUTWidget()
        Ntick = 3
        colors_red = [(int(r), 0, 0) for r in pg.np.linspace(0, 255, Ntick)]
        colors_green = [(0, int(g), 0) for g in pg.np.linspace(0, 255, Ntick)]
        colors_blue = [(0, 0, int(b)) for b in pg.np.linspace(0, 255, Ntick)]
        cmap_red = pg.ColorMap(pos=pg.np.linspace(0.0, 1.0, Ntick), color=colors_red)
        cmap_green = pg.ColorMap(pos=pg.np.linspace(0.0, 1.0, Ntick), color=colors_green)
        cmap_blue = pg.ColorMap(pos=pg.np.linspace(0.0, 1.0, Ntick), color=colors_blue)
        self.histogram_red.gradient.setColorMap(cmap_red)
        self.histogram_green.gradient.setColorMap(cmap_green)
        self.histogram_blue.gradient.setColorMap(cmap_blue)
        histo_layout.addWidget(self.histogram_red)
        histo_layout.addWidget(self.histogram_green)
        histo_layout.addWidget(self.histogram_blue)

        hsplitter.addWidget(self.histo_widget)

class ImageItem(pg.ImageItem):
    def __init__(self, image=None, **kargs):
        super(ImageItem, self).__init__(image, **kargs)

    def getHistogram(self, bins='auto', step='auto', targetImageSize=200, targetHistogramSize=500, **kwds):
        """Returns x and y arrays containing the histogram values for the current image.
        For an explanation of the return format, see numpy.histogram().

        The *step* argument causes pixels to be skipped when computing the histogram to save time.
        If *step* is 'auto', then a step is chosen such that the analyzed data has
        dimensions roughly *targetImageSize* for each axis.

        The *bins* argument and any extra keyword arguments are passed to
        np.histogram(). If *bins* is 'auto', then a bin number is automatically
        chosen based on the image characteristics:

        * Integer images will have approximately *targetHistogramSize* bins,
          with each bin having an integer width.
        * All other types will have *targetHistogramSize* bins.

        This method is also used when automatically computing levels.
        """
        if self.image is None:
            return None, None
        if step == 'auto':
            step = (int(np.ceil(self.image.shape[0] / targetImageSize)),
                    int(np.ceil(self.image.shape[1] / targetImageSize)))
        if np.isscalar(step):
            step = (step, step)
        stepData = self.image[::step[0], ::step[1]]

        if bins == 'auto':
            try:
                if stepData.dtype.kind in "ui":
                    mn = stepData.min()
                    mx = stepData.max()
                    step = np.ceil((mx - mn) / 500.)
                    bins = np.arange(mn, mx + 1.01 * step, step, dtype=np.int)
                    if len(bins) == 0:
                        bins = [mn, mx]
            except:
                bins = 500
            else:
                bins = 500

        kwds['bins'] = bins
        stepData = stepData[np.isfinite(stepData)]
        hist = np.histogram(stepData, **kwds)

        return hist[1][:-1], hist[0]



class ImageWidget(pg.GraphicsLayoutWidget):
    """this gives a layout to add imageitems.
    """
    def __init__(self, parent = None):
        QLocale.setDefault(QLocale(QLocale.English, QLocale.UnitedStates))
        super(ImageWidget, self).__init__(parent)
        self.setupUI()

    def setupUI(self):
        layout = QtWidgets.QGridLayout()
        #set viewer area
        self.scene_obj = self.scene()
        self.view = View_cust()
        self.plotitem = pg.PlotItem(viewBox=self.view)
        self.plotItem = self.plotitem #for backcompatibility
        self.plotitem.vb.setAspectLocked(lock=True, ratio=1)
        self.setCentralItem(self.plotitem)


class View_cust(pg.ViewBox):
    """Custom ViewBox used to enable other properties compared to parent class: pg.ViewBox

    """
    sig_double_clicked = pyqtSignal(float, float)

    def __init__(self, parent=None, border=None, lockAspect=False, enableMouse=True, invertY=False,
                 enableMenu=True, name=None, invertX=False):
        super(View_cust, self).__init__(parent, border, lockAspect, enableMouse, invertY, enableMenu, name,
                                        invertX)

    def mouseClickEvent(self, ev):
        if ev.button() == QtCore.Qt.RightButton and self.menuEnabled():
            ev.accept()
            self.raiseContextMenu(ev)
        if ev.double():
            pos = self.mapToView(ev.pos())
            self.sig_double_clicked.emit(pos.x(), pos.y())


class AxisItem_Scaled(pg.AxisItem):
    """
    Subclass of pg.AxisItem enabling scaling of the tick values with respect to the linked viewbox
    """

    def __init__(self, orientation, scaling=1, offset=0, pen=None, linkView=None, parent=None, maxTickLength=-5,
                 showValues=True):
        """
        ==============  ===============================================================
        **Arguments:**
        orientation     one of 'left', 'right', 'top', or 'bottom'
        scaling         multiplicative coeff applied to the ticks
        offset          offset applied to the ticks after scaling
        maxTickLength   (px) maximum length of ticks to draw. Negative values draw
                        into the plot, positive values draw outward.
        linkView        (ViewBox) causes the range of values displayed in the axis
                        to be linked to the visible range of a ViewBox.
        showValues      (bool) Whether to display values adjacent to ticks
        pen             (QPen) Pen used when drawing ticks.
        ==============  ===============================================================
        """
        pg.AxisItem.__init__(self, orientation, pen, linkView, parent, maxTickLength, showValues)
        self.scaling = scaling
        self.offset = offset

    def linkedViewChanged(self, view, newRange=None):
        if self.orientation in ['right', 'left']:
            if newRange is None:
                newRange = [pos * self.scaling + self.offset for pos in view.viewRange()[1]]
            else:
                newRange = [pos * self.scaling + self.offset for pos in newRange]

            if view.yInverted():
                self.setRange(*newRange[::-1])
            else:
                self.setRange(*newRange)
        else:
            if newRange is None:
                newRange = [pos * self.scaling + self.offset for pos in view.viewRange()[0]]
            else:
                newRange = [pos * self.scaling + self.offset for pos in newRange]
            if view.xInverted():
                self.setRange(*newRange[::-1])
            else:
                self.setRange(*newRange)


if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    form = QtWidgets.QWidget()
    prog = Viewer2DBasic(form)

    form.show()
    sys.exit(app.exec_())


from PyQt5 import QtGui, QtWidgets, QtCore
from PyQt5.QtCore import QThread
import sys
import os
import random

import pyqtgraph.parametertree.parameterTypes as pTypes
from pyqtgraph.parametertree import Parameter, ParameterTree
import pymodaq.daq_utils.custom_parameter_tree as custom_tree# to be placed after importing Parameter
from pyqtgraph.parametertree.Parameter import registerParameterType

from pymodaq.daq_utils.daq_utils import select_file


class PresetScalableGroupMove( pTypes.GroupParameter):
    """
        |

        ================ =============
        **Attributes**    **Type**
        *opts*            dictionnary
        ================ =============

        See Also
        --------
        hardware.DAQ_Move_Stage_type
    """
    def __init__(self, **opts):
        opts['type'] = 'groupmoveover'
        opts['addText'] = "Add"
        opts['addList'] = opts['movelist']
        pTypes.GroupParameter.__init__(self, **opts)

    def addNew(self, name):
        """
            Add a child.

            =============== ===========
            **Parameters**   **Type**
            *typ*            string
            =============== ===========
        """
        childnames=[par.name() for par in self.children()]
        if childnames==[]:
            newindex=0
        else:
            newindex=len(childnames)

        child={'title': name ,'name': 'move{:02.0f}'.format(newindex), 'type': 'group', 'removable': True, 'children': [
                {'title': 'Move if overshoot?:' , 'name': 'move_overshoot', 'type': 'bool', 'value': True},
                {'title': 'Position:', 'name': 'position', 'type': 'float', 'value': 0}],'removable':True, 'renamable':False}

        self.addChild(child)
registerParameterType('groupmoveover', PresetScalableGroupMove, override=True)

class PresetScalableGroupDet( pTypes.GroupParameter):
    """
        =============== ==============
        **Attributes**    **Type**
        *opts*            dictionnary
        *options*         string list
        =============== ==============

        See Also
        --------
        pymodaq.daq_utils.daq_utils.make_enum
    """
    def __init__(self, **opts):
        opts['type'] = 'groupdetover'
        opts['addText'] = "Add"
        opts['addList'] = opts['detlist']
        opts['movelist'] = opts['movelist']

        pTypes.GroupParameter.__init__(self, **opts)

    def addNew(self, name):
        """
            Add a child.

            =============== ===========  ================
            **Parameters**    **Type**   **Description*
            *typ*             string     the viewer name
            =============== ===========  ================
        """
        try:
            childnames=[par.name() for par in self.children()]
            if childnames==[]:
                newindex=0
            else:
                newindex=len(childnames)

            child={'title': name ,'name': 'det{:02.0f}'.format(newindex), 'type': 'group', 'children': [
                    {'title': 'Trig overshoot?:' , 'name': 'trig_overshoot', 'type': 'bool', 'value': True},
                    {'title': 'Overshoot value:', 'name': 'overshoot_value', 'type': 'float', 'value': 20},
                    {'title': 'Triggered Moves:', 'name': 'params', 'type': 'groupmoveover', 'movelist': self.opts['movelist']}],'removable':True, 'renamable':False}

            self.addChild(child)
        except Exception as e:
            print(str(e))
registerParameterType('groupdetover', PresetScalableGroupDet, override=True)


#check if overshoot_configurations directory exists on the drive
from pymodaq.daq_utils.daq_utils import get_set_local_dir
local_path = get_set_local_dir()
overshoot_path= os.path.join(local_path, 'overshoot_configurations')
if not os.path.isdir(overshoot_path):
    os.makedirs(overshoot_path)


class OvershootManager():
    def __init__(self,msgbox=False,det_modules=[],move_modules=[]):

        self.overshoot_params=None
        self.det_modules = det_modules
        self.move_modules = move_modules

        if msgbox:
            msgBox=QtWidgets.QMessageBox()
            msgBox.setText("Overshoot Manager?");
            msgBox.setInformativeText("What do you want to do?");
            cancel_button = msgBox.addButton(QtWidgets.QMessageBox.Cancel)
            new_button=msgBox.addButton("New", QtWidgets.QMessageBox.ActionRole)
            modify_button=msgBox.addButton('Modify', QtWidgets.QMessageBox.AcceptRole)
            msgBox.setDefaultButton(QtWidgets.QMessageBox.Cancel)
            ret = msgBox.exec()

            if msgBox.clickedButton() == new_button:
                self.set_new_overshoot()

            elif msgBox.clickedButton() == modify_button:
                path = select_file(start_path=overshoot_path,save=False, ext='xml')
                if path != '':
                    self.set_file_overshoot(str(path))
            else: #cancel
                pass

    def set_file_overshoot(self, filename,show=True):
        """

        """
        children = custom_tree.XML_file_to_parameter(filename)
        self.overshoot_params = Parameter.create(title='Overshoot', name='Overshoot', type='group', children=children)
        if show:
            self.show_overshoot()


    def set_new_overshoot(self, file = None):
        if file is None:
            file = 'overshoot_default'
        param = [{'title': 'Filename:', 'name': 'filename', 'type': 'str', 'value': file}]
        params_det = [{'title': 'Detectors:', 'name': 'Detectors','type': 'groupdetover', 'detlist': self.det_modules, 'movelist': self.move_modules}]  # [PresetScalableGroupDet(name="Detectors")]
        self.overshoot_params=Parameter.create(title='Preset', name='Preset', type='group', children=param+params_det)

        self.show_overshoot()

    def show_overshoot(self):
        """

        """
        dialog = QtWidgets.QDialog()
        vlayout = QtWidgets.QVBoxLayout()
        tree = ParameterTree()
        tree.setMinimumWidth(400)
        tree.setMinimumHeight(500)
        tree.setParameters(self.overshoot_params, showTop=False)

        vlayout.addWidget(tree)
        dialog.setLayout(vlayout)
        buttonBox = QtWidgets.QDialogButtonBox(parent=dialog)

        buttonBox.addButton('Save', buttonBox.AcceptRole)
        buttonBox.accepted.connect(dialog.accept)
        buttonBox.addButton('Cancel', buttonBox.RejectRole)
        buttonBox.rejected.connect(dialog.reject)

        vlayout.addWidget(buttonBox)
        dialog.setWindowTitle('Fill in information about this preset')
        res = dialog.exec()

        if res == dialog.Accepted:
            # save preset parameters in a xml file
            #start = os.path.split(os.path.split(os.path.realpath(__file__))[0])[0]
            #start = os.path.join("..",'daq_scan')
            custom_tree.parameter_to_xml_file(self.overshoot_params, os.path.join(overshoot_path,
                                                                               self.overshoot_params.child(
                                                                                   ('filename')).value()))


if __name__ == '__main__':
    app = QtWidgets.QApplication(sys.argv)
    prog = OvershootManager(True,['det camera','det current'],['Move X','Move Y'])

    sys.exit(app.exec_())


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'daq_gui_settings.ui'
#
# Created by: PyQt5 UI code generator 5.11.3
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(636, 590)
        self.gridLayout = QtWidgets.QGridLayout(Form)
        self.gridLayout.setContentsMargins(9, 9, 9, 9)
        self.gridLayout.setSpacing(6)
        self.gridLayout.setObjectName("gridLayout")
        self.settings_layout = QtWidgets.QVBoxLayout()
        self.settings_layout.setObjectName("settings_layout")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.title_label = QtWidgets.QLabel(Form)
        self.title_label.setMaximumSize(QtCore.QSize(16777215, 30))
        font = QtGui.QFont()
        font.setFamily("Tahoma")
        font.setPointSize(14)
        font.setBold(True)
        font.setItalic(True)
        font.setWeight(75)
        self.title_label.setFont(font)
        self.title_label.setAlignment(QtCore.Qt.AlignCenter)
        self.title_label.setObjectName("title_label")
        self.verticalLayout_4.addWidget(self.title_label)
        self.command_widget = QtWidgets.QWidget(Form)
        self.command_widget.setMaximumSize(QtCore.QSize(16777215, 30))
        self.command_widget.setObjectName("command_widget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.command_widget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.grab_pb = QtWidgets.QPushButton(self.command_widget)
        self.grab_pb.setText("")
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/run2.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.grab_pb.setIcon(icon)
        self.grab_pb.setCheckable(True)
        self.grab_pb.setObjectName("grab_pb")
        self.horizontalLayout_2.addWidget(self.grab_pb)
        self.single_pb = QtWidgets.QPushButton(self.command_widget)
        self.single_pb.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/snap.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.single_pb.setIcon(icon1)
        self.single_pb.setCheckable(False)
        self.single_pb.setObjectName("single_pb")
        self.horizontalLayout_2.addWidget(self.single_pb)
        self.stop_pb = QtWidgets.QPushButton(self.command_widget)
        self.stop_pb.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/stop.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.stop_pb.setIcon(icon2)
        self.stop_pb.setCheckable(False)
        self.stop_pb.setObjectName("stop_pb")
        self.horizontalLayout_2.addWidget(self.stop_pb)
        self.save_current_pb = QtWidgets.QPushButton(self.command_widget)
        self.save_current_pb.setText("")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/SaveAs.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.save_current_pb.setIcon(icon3)
        self.save_current_pb.setObjectName("save_current_pb")
        self.horizontalLayout_2.addWidget(self.save_current_pb)
        self.save_new_pb = QtWidgets.QPushButton(self.command_widget)
        self.save_new_pb.setText("")
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/Snap&Save.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.save_new_pb.setIcon(icon4)
        self.save_new_pb.setObjectName("save_new_pb")
        self.horizontalLayout_2.addWidget(self.save_new_pb)
        self.load_data_pb = QtWidgets.QPushButton(self.command_widget)
        self.load_data_pb.setText("")
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/Open.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.load_data_pb.setIcon(icon5)
        self.load_data_pb.setObjectName("load_data_pb")
        self.horizontalLayout_2.addWidget(self.load_data_pb)
        self.settings_pb = QtWidgets.QPushButton(self.command_widget)
        self.settings_pb.setText("")
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/HLM.ico"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.settings_pb.setIcon(icon6)
        self.settings_pb.setCheckable(True)
        self.settings_pb.setObjectName("settings_pb")
        self.horizontalLayout_2.addWidget(self.settings_pb)
        self.update_com_pb = QtWidgets.QPushButton(self.command_widget)
        self.update_com_pb.setText("")
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/Refresh2.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.update_com_pb.setIcon(icon7)
        self.update_com_pb.setCheckable(False)
        self.update_com_pb.setObjectName("update_com_pb")
        self.horizontalLayout_2.addWidget(self.update_com_pb)
        self.navigator_pb = QtWidgets.QPushButton(self.command_widget)
        self.navigator_pb.setText("")
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/Select_24.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.navigator_pb.setIcon(icon8)
        self.navigator_pb.setObjectName("navigator_pb")
        self.horizontalLayout_2.addWidget(self.navigator_pb)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.verticalLayout_4.addWidget(self.command_widget)
        self.settings_widget = QtWidgets.QWidget(Form)
        self.settings_widget.setObjectName("settings_widget")
        self.settings_layout_2 = QtWidgets.QVBoxLayout(self.settings_widget)
        self.settings_layout_2.setContentsMargins(0, 0, 0, 0)
        self.settings_layout_2.setSpacing(0)
        self.settings_layout_2.setObjectName("settings_layout_2")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setContentsMargins(-1, 0, -1, -1)
        self.gridLayout_3.setSpacing(6)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label_3 = QtWidgets.QLabel(self.settings_widget)
        self.label_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_3.setObjectName("label_3")
        self.gridLayout_3.addWidget(self.label_3, 1, 0, 1, 3)
        self.DAQ_type_combo = QtWidgets.QComboBox(self.settings_widget)
        self.DAQ_type_combo.setObjectName("DAQ_type_combo")
        self.DAQ_type_combo.addItem("")
        self.DAQ_type_combo.addItem("")
        self.DAQ_type_combo.addItem("")
        self.gridLayout_3.addWidget(self.DAQ_type_combo, 0, 3, 1, 3)
        self.label_4 = QtWidgets.QLabel(self.settings_widget)
        self.label_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_4.setObjectName("label_4")
        self.gridLayout_3.addWidget(self.label_4, 0, 0, 1, 3)
        self.Quit_pb = QtWidgets.QPushButton(self.settings_widget)
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/close2.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.Quit_pb.setIcon(icon9)
        self.Quit_pb.setObjectName("Quit_pb")
        self.gridLayout_3.addWidget(self.Quit_pb, 4, 0, 1, 2)
        self.Detector_type_combo = QtWidgets.QComboBox(self.settings_widget)
        self.Detector_type_combo.setObjectName("Detector_type_combo")
        self.gridLayout_3.addWidget(self.Detector_type_combo, 1, 3, 1, 3)
        self.load_settings_pb = QtWidgets.QPushButton(self.settings_widget)
        self.load_settings_pb.setIcon(icon5)
        self.load_settings_pb.setObjectName("load_settings_pb")
        self.gridLayout_3.addWidget(self.load_settings_pb, 7, 0, 3, 3)
        self.save_settings_pb = QtWidgets.QPushButton(self.settings_widget)
        self.save_settings_pb.setIcon(icon3)
        self.save_settings_pb.setObjectName("save_settings_pb")
        self.gridLayout_3.addWidget(self.save_settings_pb, 7, 3, 3, 3)
        self.take_bkg_cb = QtWidgets.QCheckBox(self.settings_widget)
        self.take_bkg_cb.setObjectName("take_bkg_cb")
        self.gridLayout_3.addWidget(self.take_bkg_cb, 5, 3, 2, 3)
        self.do_bkg_cb = QtWidgets.QCheckBox(self.settings_widget)
        self.do_bkg_cb.setObjectName("do_bkg_cb")
        self.gridLayout_3.addWidget(self.do_bkg_cb, 5, 0, 2, 3)
        self.Ini_state_LED = QLED(self.settings_widget)
        self.Ini_state_LED.setObjectName("Ini_state_LED")
        self.gridLayout_3.addWidget(self.Ini_state_LED, 4, 4, 1, 1)
        self.data_ready_led = QLED(self.settings_widget)
        self.data_ready_led.setObjectName("data_ready_led")
        self.gridLayout_3.addWidget(self.data_ready_led, 4, 5, 1, 1)
        self.IniDet_pb = QtWidgets.QPushButton(self.settings_widget)
        self.IniDet_pb.setCheckable(True)
        self.IniDet_pb.setChecked(False)
        self.IniDet_pb.setObjectName("IniDet_pb")
        self.gridLayout_3.addWidget(self.IniDet_pb, 4, 2, 1, 2)
        self.settings_layout_2.addLayout(self.gridLayout_3)
        self.verticalLayout_4.addWidget(self.settings_widget)
        self.settings_layout.addLayout(self.verticalLayout_4)
        self.gridLayout.addLayout(self.settings_layout, 0, 0, 1, 1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.title_label.setText(_translate("Form", "DAQ Title"))
        self.grab_pb.setToolTip(_translate("Form", "Grab"))
        self.single_pb.setToolTip(_translate("Form", "Single Grab"))
        self.stop_pb.setToolTip(_translate("Form", "Single Grab"))
        self.save_current_pb.setToolTip(_translate("Form", "Save Current Data"))
        self.save_new_pb.setToolTip(_translate("Form", "Save New Data"))
        self.settings_pb.setToolTip(_translate("Form", "Open Settings"))
        self.update_com_pb.setToolTip(_translate("Form", "Refresh Hardware"))
        self.navigator_pb.setToolTip(_translate("Form", "Send current data to navigator"))
        self.label_3.setText(_translate("Form", "Detector:"))
        self.DAQ_type_combo.setToolTip(_translate("Form", "Detector Type"))
        self.DAQ_type_combo.setItemText(0, _translate("Form", "DAQ0D"))
        self.DAQ_type_combo.setItemText(1, _translate("Form", "DAQ1D"))
        self.DAQ_type_combo.setItemText(2, _translate("Form", "DAQ2D"))
        self.label_4.setText(_translate("Form", "DAQ type:"))
        self.Quit_pb.setToolTip(_translate("Form", "quit and close the viewer"))
        self.Quit_pb.setText(_translate("Form", "Quit"))
        self.Detector_type_combo.setToolTip(_translate("Form", "Stage Type"))
        self.load_settings_pb.setToolTip(_translate("Form", "Load settings"))
        self.load_settings_pb.setText(_translate("Form", "Sett."))
        self.save_settings_pb.setToolTip(_translate("Form", "save settings"))
        self.save_settings_pb.setText(_translate("Form", "Sett."))
        self.take_bkg_cb.setText(_translate("Form", "Take Bkg"))
        self.do_bkg_cb.setText(_translate("Form", "Do Bkg sub."))
        self.Ini_state_LED.setToolTip(_translate("Form", "Green when controller is initialized"))
        self.Ini_state_LED.setText(_translate("Form", "TextLabel"))
        self.data_ready_led.setToolTip(_translate("Form", "Green when data ready"))
        self.data_ready_led.setText(_translate("Form", "TextLabel"))
        self.IniDet_pb.setToolTip(_translate("Form", "To initialize the detector"))
        self.IniDet_pb.setText(_translate("Form", "Ini. Det."))

from pymodaq.daq_utils.plotting.QLED.qled import QLED
from pymodaq.QtDesigner_Ressources import QtDesigner_ressources_rc

if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'trace1D_selection_GUI.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(397, 473)
        self.gridLayout_2 = QtWidgets.QGridLayout(Dialog)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.delimiter_combo = QtWidgets.QComboBox(Dialog)
        self.delimiter_combo.setObjectName("delimiter_combo")
        self.delimiter_combo.addItem("")
        self.delimiter_combo.addItem("")
        self.delimiter_combo.addItem("")
        self.gridLayout.addWidget(self.delimiter_combo, 4, 1, 1, 1)
        self.retry_pb = QtWidgets.QPushButton(Dialog)
        self.retry_pb.setObjectName("retry_pb")
        self.gridLayout.addWidget(self.retry_pb, 1, 1, 2, 1)
        self.label_5 = QtWidgets.QLabel(Dialog)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 6, 0, 1, 1)
        self.col_x_sb = QtWidgets.QSpinBox(Dialog)
        self.col_x_sb.setObjectName("col_x_sb")
        self.gridLayout.addWidget(self.col_x_sb, 5, 1, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 1, 2, 3, 1)
        self.label_3 = QtWidgets.QLabel(Dialog)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 4, 0, 1, 1)
        self.label_4 = QtWidgets.QLabel(Dialog)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 5, 0, 1, 1)
        self.header_sb = QtWidgets.QSpinBox(Dialog)
        self.header_sb.setObjectName("header_sb")
        self.gridLayout.addWidget(self.header_sb, 3, 1, 1, 1)
        self.label_2 = QtWidgets.QLabel(Dialog)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 3, 0, 1, 1)
        self.label = QtWidgets.QLabel(Dialog)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 1, 0, 1, 1)
        self.col_y_sb = QtWidgets.QSpinBox(Dialog)
        self.col_y_sb.setProperty("value", 1)
        self.col_y_sb.setObjectName("col_y_sb")
        self.gridLayout.addWidget(self.col_y_sb, 6, 1, 1, 1)
        self.data_name_edit = QtWidgets.QLineEdit(Dialog)
        self.data_name_edit.setObjectName("data_name_edit")
        self.gridLayout.addWidget(self.data_name_edit, 2, 0, 1, 1)
        self.graphicsView = PlotWidget(Dialog)
        self.graphicsView.setObjectName("graphicsView")
        self.gridLayout.addWidget(self.graphicsView, 0, 0, 1, 3)
        self.verticalLayout_2.addLayout(self.gridLayout)
        self.gridLayout_2.addLayout(self.verticalLayout_2, 0, 0, 1, 1)
        self.graphicsView.raise_()

        self.retranslateUi(Dialog)
        self.buttonBox.accepted.connect(Dialog.accept)
        self.buttonBox.rejected.connect(Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.delimiter_combo.setItemText(0, _translate("Dialog", "tab"))
        self.delimiter_combo.setItemText(1, _translate("Dialog", "comma"))
        self.delimiter_combo.setItemText(2, _translate("Dialog", "space"))
        self.retry_pb.setText(_translate("Dialog", "Retry\n"
"Loading!"))
        self.label_5.setText(_translate("Dialog", "Col y:"))
        self.label_3.setText(_translate("Dialog", "Delimiter:"))
        self.label_4.setText(_translate("Dialog", "Col x:"))
        self.label_2.setText(_translate("Dialog", "Header lines:"))
        self.label.setText(_translate("Dialog", "Save 1D trace in tdms as:"))

from pyqtgraph import PlotWidget

if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Dialog = QtWidgets.QDialog()
    ui = Ui_Dialog()
    ui.setupUi(Dialog)
    Dialog.show()
    sys.exit(app.exec_())




from PyQt5 import QtWidgets, QtCore, QtGui
from PyQt5.QtCore import QObject
from pymodaq.daq_utils.plotting.viewer0D.viewer0D_main import Viewer0D
import sys


class LCD(QObject):

    def __init__(self, parent, **kwargs):
        super(LCD, self).__init__()
        self.Nvals = None
        self.labels = None
        self.parent = parent
        self.viewer0D = None
        self.digits = 3
        self.setupui(**kwargs)

    def setvalues(self, values):
        """
        display values on lcds
        Parameters
        ----------
        values: list of list of one element

        Returns
        -------

        """
        while len(values) < self.Nvals:
            values.append(0)
        if len(values) > self.Nvals:
            values = values[:self.Nvals]
        vals = []
        for ind, val in enumerate(values):
            self.lcds[ind].display(val[0])
            vals.append(val)
        self.viewer0D.show_data(vals)

    def setupui(self, **kwargs):
        if 'digits' in kwargs:
            self.digits = kwargs['digits']
        if 'Nvals' in kwargs:
            self.Nvals = kwargs['Nvals']
        else:
            self.Nvals = 1
        if 'labels' in kwargs:
            self.labels = kwargs['labels']
        else:
            self.labels = ['CH{:d}'.format(ind) for ind in range(self.Nvals)]

        while len(self.labels) < self.Nvals:
            self.labels.append('')

        vlayout = QtWidgets.QVBoxLayout()
        hsplitter = QtWidgets.QSplitter()
        vlayout.addWidget(hsplitter)
        self.parent.setLayout(vlayout)
        form = QtWidgets.QWidget()
        self.viewer0D = Viewer0D(form)

        vlayout = QtWidgets.QVBoxLayout()

        lcd_layouts = []
        labels = []
        self.lcds = []


        for ind in range(self.Nvals):
            lcd_layouts.append(QtWidgets.QVBoxLayout())
            labels.append(QtWidgets.QLabel(self.labels[ind]))
            labels[-1].setSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
            self.lcds.append(QtWidgets.QLCDNumber())
            self.lcds[-1].setDigitCount(self.digits)
            lcd_layouts[-1].addWidget(labels[-1])
            lcd_layouts[-1].addWidget(self.lcds[-1])
            vlayout.addLayout(lcd_layouts[-1])

            if ind != self.Nvals-1:
                hFrame = QtWidgets.QFrame()
                hFrame.setFrameShape(QtWidgets.QFrame.HLine)
                vlayout.addWidget(hFrame)


        lcd_widget = QtWidgets.QWidget()
        lcd_widget.setLayout(vlayout)
        hsplitter.addWidget(lcd_widget)
        hsplitter.addWidget(form)
        self.parent.resize(800,500)
        hsplitter.setSizes([400, 300])


if __name__ == '__main__':
    from pymodaq.daq_utils.daq_utils import gauss1D
    import numpy as np
    x=np.linspace(0,200,201)
    y1=gauss1D(x,75,25)
    y2=gauss1D(x,120,50,2)
    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()


    prog = LCD(Form, Nvals=2)
    Form.show()
    for ind,data in enumerate(y1):
        prog.setvalues([data])
        QtWidgets.QApplication.processEvents()
    sys.exit(app.exec_())




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'daq_scan_gui_v2.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(713, 485)
        self.gridLayout_4 = QtWidgets.QGridLayout(Form)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.splitter = QtWidgets.QSplitter(Form)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.horizontalLayoutWidget = QtWidgets.QWidget(self.splitter)
        self.horizontalLayoutWidget.setObjectName("horizontalLayoutWidget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.horizontalLayoutWidget)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.settings_layout = QtWidgets.QVBoxLayout()
        self.settings_layout.setObjectName("settings_layout")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.set_ini_positions_pb = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.set_ini_positions_pb.setObjectName("set_ini_positions_pb")
        self.gridLayout_3.addWidget(self.set_ini_positions_pb, 2, 1, 1, 1)
        self.quit_pb = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/close2.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.quit_pb.setIcon(icon)
        self.quit_pb.setObjectName("quit_pb")
        self.gridLayout_3.addWidget(self.quit_pb, 0, 0, 1, 1)
        self.line = QtWidgets.QFrame(self.horizontalLayoutWidget)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout_3.addWidget(self.line, 1, 0, 1, 1)
        self.set_scan_pb = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.set_scan_pb.setObjectName("set_scan_pb")
        self.gridLayout_3.addWidget(self.set_scan_pb, 2, 0, 1, 1)
        self.start_scan_pb = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.start_scan_pb.setText("")
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/run2.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.start_scan_pb.setIcon(icon1)
        self.start_scan_pb.setObjectName("start_scan_pb")
        self.gridLayout_3.addWidget(self.start_scan_pb, 3, 0, 1, 1)
        self.stop_scan_pb = QtWidgets.QPushButton(self.horizontalLayoutWidget)
        self.stop_scan_pb.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/icons/Icon_Library/stop.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.stop_scan_pb.setIcon(icon2)
        self.stop_scan_pb.setObjectName("stop_scan_pb")
        self.gridLayout_3.addWidget(self.stop_scan_pb, 3, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_3.addItem(spacerItem, 0, 2, 4, 1)
        self.settings_layout.addLayout(self.gridLayout_3)
        self.horizontalLayout.addLayout(self.settings_layout)
        self.tabWidget = QtWidgets.QTabWidget(self.splitter)
        self.tabWidget.setTabsClosable(False)
        self.tabWidget.setTabBarAutoHide(False)
        self.tabWidget.setObjectName("tabWidget")
        self.tab_plot1D = QtWidgets.QWidget()
        self.tab_plot1D.setObjectName("tab_plot1D")
        self.gridLayout = QtWidgets.QGridLayout(self.tab_plot1D)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.scan1D_layout = QtWidgets.QVBoxLayout()
        self.scan1D_layout.setObjectName("scan1D_layout")
        self.gridLayout.addLayout(self.scan1D_layout, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tab_plot1D, "")
        self.tab_plot2D = QtWidgets.QWidget()
        self.tab_plot2D.setObjectName("tab_plot2D")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.tab_plot2D)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.scan2D_layout = QtWidgets.QVBoxLayout()
        self.scan2D_layout.setObjectName("scan2D_layout")
        self.gridLayout_2.addLayout(self.scan2D_layout, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tab_plot2D, "")
        self.tab_navigator = QtWidgets.QWidget()
        self.tab_navigator.setObjectName("tab_navigator")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.tab_navigator)
        self.gridLayout_5.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_5.setSpacing(0)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.navigator_layout = QtWidgets.QVBoxLayout()
        self.navigator_layout.setObjectName("navigator_layout")
        self.gridLayout_5.addLayout(self.navigator_layout, 0, 0, 1, 1)
        self.tabWidget.addTab(self.tab_navigator, "")
        self.gridLayout_4.addWidget(self.splitter, 0, 0, 1, 1)
        self.StatusBarLayout = QtWidgets.QHBoxLayout()
        self.StatusBarLayout.setObjectName("StatusBarLayout")
        self.gridLayout_4.addLayout(self.StatusBarLayout, 1, 0, 1, 1)

        self.retranslateUi(Form)
        self.tabWidget.setCurrentIndex(2)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.set_ini_positions_pb.setText(_translate("Form", "Set Ini Positions"))
        self.quit_pb.setToolTip(_translate("Form", "Stop Scan"))
        self.quit_pb.setText(_translate("Form", "Quit"))
        self.set_scan_pb.setText(_translate("Form", "Set Scan"))
        self.start_scan_pb.setToolTip(_translate("Form", "Start Scan"))
        self.stop_scan_pb.setToolTip(_translate("Form", "Stop Scan"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_plot1D), _translate("Form", "1D plot"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_plot2D), _translate("Form", "2D plot"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_navigator), _translate("Form", "Navigator"))

from pymodaq.QtDesigner_Ressources import QtDesigner_ressources_rc

if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    Form = QtWidgets.QWidget()
    ui = Ui_Form()
    ui.setupUi(Form)
    Form.show()
    sys.exit(app.exec_())




#%%
#%reset -f
#%%
import os
import python_lib as mylib
import matplotlib.pyplot as plt
import numpy as np
import inspect


from PyQt5 import QtWidgets
#%%

path="D:\\Data\\2016\\Vincent\\PL SiNCs_NWs"
fname = QtWidgets.QFileDialog.getOpenFileName(None, 'Choose image file',path,"image file (*.dat)")
fname=fname[0]
#%%
data_tmp=np.loadtxt(fname,delimiter='\t',skiprows=1);
xaxis=data_tmp[0,1:]
yaxis=data_tmp[1:,0]
data=data_tmp[1:,1:]
#%%



#%% plot image raw

fig=mylib.figure_docked('raw',clf=True)
ax1=plt.subplot(1,1,1)
plt.pcolormesh(xaxis,yaxis,(data), cmap='copper')
# for other choices of maps:http://matplotlib.org/examples/color/colormaps_reference.html

ax1.set_aspect('equal')
ax1.set_xlim([-2,3.5])
ax1.set_ylim([-1.3,1.2])
ax1.set_xlabel('x axis (µm)')
ax1.set_ylabel('y axis (µm)')
ax1.set_title('Lockin Magnitude cartography (µV or mV)')
plt.colorbar()

#%% saving

#mylib.add_figure_timestamp(fig,fname,FontSize=10)
(name,ext)=os.path.splitext(fname)
fig.savefig(name+'.png')

